@@ -18,7 +18,10 @@
 
 /*
  * Functions for facilitating authorization (GUI)
- * first created by Patrick Sung (Jan 2002)
+ */
+
+/*
+ * Copyright (c) 2002-2003 Patrick Sung
  */
 
 #include "common.h"
@@ -31,11 +34,8 @@
 
 #include <gtk/gtk.h>
 
-static UIN_T req_uin = NULL;
-
 void auth_sendbutton_clicked (GtkWidget *w, gpointer d);
 void auth_cancelbutton_clicked (GtkWidget *w, gpointer d);
-gboolean auth_receive_dialog (gchar *msg);
 void auth_grantbutton_clicked (GtkWidget *w, gpointer d);
 void auth_denybutton_clicked (GtkWidget *w, gpointer d);
 
@@ -47,7 +47,7 @@ auth_request_msg_box (const gchar *nick, UIN_T uin)
   GtkWidget *authlabel;
   GtkWidget *authmsg;
   GtkWidget *sendbutton;
-  gchar *str, *message;
+  gchar *str, *message = NULL;
 
 #ifdef TRACE_FUNCTION
   g_print( "auth_request_msg_box\n" );
@@ -60,16 +60,24 @@ auth_request_msg_box (const gchar *nick, UIN_T uin)
     return;
 
   authlabel = glade_xml_get_widget (authbox, "authlabel");
-  message = g_strdup_printf (_("User %s (%s) need authorization. GnomeICU is going "
+  
+
+  message = g_strdup_printf (_("User %s (%s) needs authorization. GnomeICU is going "
                                "to send an authorization message to your friend."),
                              nick ? nick : "", uin);
 
+
   str = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
                          _("Request Authorization"), message);
   gtk_label_set_markup (GTK_LABEL (authlabel), str);
   g_free (message);
   g_free (str);
 
+  str = g_strdup_printf(_("Request Authorization from %s (%s)"), 
+			    nick ? nick : "", uin);
+  gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget (authbox, "auth_request")),str);
+  g_free(str);
+
   authmsg = glade_xml_get_widget (authbox, "authmsgtext");
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (authmsg), 
 			       GTK_WRAP_WORD);
@@ -85,6 +93,9 @@ auth_request_msg_box (const gchar *nick, UIN_T uin)
                      "dlg", glade_xml_get_widget (authbox, "auth_request"));
   g_object_set_data (G_OBJECT (glade_xml_get_widget (authbox, "sendbutton")),
                      "text", authmsg);
+  g_object_set_data (G_OBJECT (glade_xml_get_widget (authbox, "sendbutton")),
+                     "uin", uin);
+  
 
   /* connect the signals for the send button and cancel button */
   glade_xml_signal_autoconnect (authbox);
@@ -121,7 +132,8 @@ auth_sendbutton_clicked (GtkWidget *w, gpointer d)
   
   msg = gtk_text_buffer_get_text(text_buffer, &start_iter, &end_iter, FALSE);
 
-  v7_ask_uin_for_auth (mainconnection, msg);
+  v7_ask_uin_for_auth (mainconnection, msg, g_object_get_data(G_OBJECT(w),
+							      "uin"));
 
   g_free (msg);
   gtk_widget_destroy (dialog);
@@ -143,23 +155,7 @@ auth_cancelbutton_clicked (GtkWidget *w, gpointer d)
 
 /* auth request received from other users, show the auth receive msg box */
 void
-auth_receive_request (UIN_T uin, const gchar *msg)
-{
-  gchar *mymsg;
-
-  g_assert(uin != NULL);
-
-  if (req_uin != NULL)
-	  g_free (req_uin);
-  req_uin = g_strdup (uin);
-  mymsg = g_strdup (msg);
-
-  /* do the auth stuff */
-  g_idle_add ((GSourceFunc)auth_receive_dialog, mymsg);
-}
-
-gboolean
-auth_receive_dialog (gchar *msg)
+auth_receive_request (UIN_T uin, const gchar *nick, const gchar *msg)
 {
   GladeXML *authbox;
   GtkWidget *grantlabel;
@@ -167,19 +163,21 @@ auth_receive_dialog (gchar *msg)
   GtkWidget *grantbutton;
   gchar *str, *message;
   GtkTextBuffer* text_buffer;
+  GSList *contact;
 
 #ifdef TRACE_FUNCTION
-  g_print( "auth_receive_dialog\n" );
+  g_print( "auth_receive_request\n" );
 #endif
 
+  g_assert(uin != NULL);
   authbox = gicu_util_open_glade_xml ("auth.glade", "auth_recv");
   if (authbox == NULL)
-    return FALSE;
+    return;
 
   grantlabel = glade_xml_get_widget (authbox, "grantlabel");
-  message = g_strdup_printf (_("User %s is requesting authorization. Do you want to "
+  message = g_strdup_printf (_("User %s (%s) is requesting authorization. Do you want to "
                                "grant the authorization so he/she can add you?"),
-                             req_uin);
+                             nick ? nick : "", uin);
 
   str = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
                          _("Received Authorization Request"), message);
@@ -187,6 +185,11 @@ auth_receive_dialog (gchar *msg)
   g_free (message);
   g_free (str);
 
+  str = g_strdup_printf(_("Authorization request from %s"), uin);
+  gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget (authbox, "auth_recv")),str);
+  g_free(str);
+  
+
   reqmsg = glade_xml_get_widget (authbox, "grantmsgtext");
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (reqmsg), 
 			       GTK_WRAP_WORD);
@@ -197,6 +200,14 @@ auth_receive_dialog (gchar *msg)
                      "dlg", glade_xml_get_widget (authbox, "auth_recv"));
   g_object_set_data (G_OBJECT (glade_xml_get_widget (authbox, "grantbutton")),
                      "dlg", glade_xml_get_widget (authbox, "auth_recv"));
+  g_object_set_data (G_OBJECT (glade_xml_get_widget (authbox, "adduserbutton")),
+                     "dlg", glade_xml_get_widget (authbox, "auth_recv"));
+  g_object_set_data (G_OBJECT ( glade_xml_get_widget (authbox, "auth_recv")),
+		     "uin", uin);
+
+  contact = Find_User (uin);
+  gtk_widget_set_sensitive (glade_xml_get_widget (authbox, "adduserbutton"),
+                            contact == NULL ? TRUE : FALSE);
 
   glade_xml_signal_autoconnect (authbox);
 
@@ -206,9 +217,6 @@ auth_receive_dialog (gchar *msg)
   g_object_unref (G_OBJECT (authbox));
 
   /* make sure we free the allocated str malloc from auth_receive_request() */
-  g_free (msg);
-
-  return FALSE;
 
   /* Additional features: (TODO) */
   /*  - also get user info from server using the UIN? */
@@ -222,25 +230,94 @@ void
 auth_grantbutton_clicked (GtkWidget *w, gpointer d)
 {
   GtkWidget *dialog;
-
+  gchar *uin;
   dialog = g_object_get_data (G_OBJECT (w), "dlg");
-
+  uin = g_object_get_data(G_OBJECT(dialog), "uin");
   if (!is_connected(GTK_WINDOW(dialog), _("You can not grant authorization while being disconnected.")))
     return;
 
+
   gtk_widget_destroy (dialog);
 
-  v7_grant_auth_request (mainconnection, req_uin);
+  v7_grant_auth_request (mainconnection, uin);
+			 
 }
 
 void
 auth_denybutton_clicked (GtkWidget *w, gpointer d)
 {
-  GtkWidget *dialog;
-
-  dialog = g_object_get_data (G_OBJECT (w), "dlg");
+  GtkWidget *dialog = g_object_get_data (G_OBJECT (w), "dlg");
 
   if (!is_connected(GTK_WINDOW(dialog), _("You can not deny authorization while being disconnected.")))
     return;
   gtk_widget_destroy (dialog);
 }
+
+static void
+add_user_response_cb (GtkDialog *d, gint response, UIN_T uin)
+{
+  GtkWidget *option;
+  GtkWidget *menu;
+  GList *item;
+  guint gid;
+
+  if (response == GTK_RESPONSE_OK) {
+    option = g_object_get_data (G_OBJECT (d), "option");
+    menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (option));
+    item = g_list_nth (GTK_MENU_SHELL (menu)->children,
+                       gtk_option_menu_get_history (GTK_OPTION_MENU (option)));
+    gid = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (item->data), "gid"));
+
+    v7_try_add_uin (mainconnection, uin, uin, gid, FALSE);
+  }
+
+  gtk_widget_destroy (GTK_WIDGET (d));
+}
+
+void
+auth_adduserbutton_clicked (GtkWidget *w, gpointer d)
+{
+  GtkWidget *dialog, *parent;
+  GtkWidget *label, *option, *menu, *box;
+  gchar *str;
+  GSList *g;
+
+  parent = g_object_get_data (G_OBJECT (w), "dlg");
+
+  dialog = gtk_dialog_new_with_buttons (_("Pick a Group"),
+                                        GTK_WINDOW (parent),
+                                        GTK_DIALOG_DESTROY_WITH_PARENT,
+                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                        GTK_STOCK_OK, GTK_RESPONSE_OK,
+                                        NULL);
+
+  box = gtk_hbox_new (FALSE, 5);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), box);
+
+  str = g_strdup_printf ("%s", _("Pick a group for this user:"));
+  label = gtk_label_new (str);
+  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 3);
+  g_free (str);
+
+  option = gtk_option_menu_new ();
+  gtk_box_pack_start (GTK_BOX (box), option, FALSE, FALSE, 3);
+  menu = gtk_menu_new ();
+
+  for (g = Groups; g != NULL; g = g->next) {
+    label = gtk_menu_item_new_with_label (((GroupInfo *)g->data)->name);
+    g_object_set_data (G_OBJECT (label), "gid",
+                       GUINT_TO_POINTER ((guint)((GroupInfo *)g->data)->gid));
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), label);
+  }
+
+  gtk_option_menu_set_menu (GTK_OPTION_MENU (option), menu);
+  gtk_option_menu_set_history (GTK_OPTION_MENU (option), 0);
+
+  g_object_set_data (G_OBJECT (dialog), "option", option);
+
+  g_signal_connect (G_OBJECT (dialog), "response",
+                    G_CALLBACK (add_user_response_cb), 
+		    g_object_get_data(G_OBJECT(parent), "uin"));
+
+  gtk_widget_show_all (dialog);
+}
