@@ -27,6 +27,6 @@
 #include "datatype.h"
 
 void auth_request_msg_box (const gchar *nick, UIN_T uin);
-void auth_receive_request (UIN_T uin, const gchar *msg);
+void auth_receive_request (UIN_T uin, const gchar *nick, const gchar *msg);
 
 #endif /* __AUTH_H__ */
