@@ -1150,10 +1150,9 @@ static void perms_save()
 
   perms_changed = FALSE;
 
-  /* save to disk ? 
+  Save_RC();
+
 
-  save_user_permsinfo();
-*/
 }
 
 void password_save()
