@@ -21,15 +21,9 @@
 static gboolean
 detach_window_click( GtkWidget *widget, GdkEventButton *event, CONTACT_PTR contact )
 {
-	GSList *contact_list = NULL;
-
 	if( event == NULL )
 	       return FALSE;
 
-#ifdef TRACE_FUNCTION
-	g_print( "detach_window_click\n" );
-#endif
-
 	if( event->type == GDK_KEY_PRESS )
 	{
  		if( ((GdkEventKey*)event)->keyval != GDK_Return && ((GdkEventKey*)event)->keyval != ' ' )
@@ -51,17 +45,16 @@ detach_window_click( GtkWidget *widget, GdkEventButton *event, CONTACT_PTR conta
 		return TRUE;
 	}
 
-	contact_list = g_slist_find( Contacts, contact );
-
 	if( g_slist_length( contact->stored_messages ) )
-		show_contact_message( contact_list );
+		show_contact_message (contact);
 	else
 	  if (contact->msg_dlg_xml != NULL) {
 	    gtk_widget_show_all(gtk_bin_get_child(GTK_BIN(glade_xml_get_widget( contact->msg_dlg_xml, "message_dialog"))));
+	    gtk_widget_grab_focus(glade_xml_get_widget( contact->msg_dlg_xml, "input"));
 	    gtk_window_present(GTK_WINDOW(glade_xml_get_widget( contact->msg_dlg_xml, "message_dialog")));
 	  }
 	  else
-	    open_message_dlg_with_message (contact_list, 0, NULL);
+	    open_message_dlg_with_message (contact, NULL);
 
 	return TRUE;
 }
@@ -130,5 +123,5 @@ void detach_contact( GtkWidget *widget, gpointer data )
     gtk_window_set_resizable( GTK_WINDOW( window ), FALSE );
     gtk_window_stick( GTK_WINDOW( window ) );
 
-    Shortcut_Update( contact );
+    Shortcut_Update (contact);
 }
