@@ -11,6 +11,10 @@
 #include "user_popup.h"
 #include "util.h"
 
+#include <string.h>
+#include <gtk/gtk.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+
 enum {
   TARGET_STRING
 };
@@ -42,10 +46,10 @@ static void contact_list_dnd_drop(GtkWidget *widget, GdkDragContext *context,
 /*** Global functions ***/
 void init_contact_list_drag_drop(GtkWidget *list)
 {
-  gtk_signal_connect(GTK_OBJECT(list),
-		     "drag_data_received",
-		     GTK_SIGNAL_FUNC(contact_list_dnd_drop),
-		     NULL);
+  g_signal_connect(G_OBJECT(list),
+		   "drag_data_received",
+		   G_CALLBACK(contact_list_dnd_drop),
+		   NULL);
   gtk_drag_dest_set(GTK_WIDGET(list),
 		    GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
 		    contact_list_drop_types, n_contact_list_drop_types,
@@ -72,7 +76,7 @@ static void contact_list_dnd_drop(GtkWidget *widget,
   CONTACT_PTR contact;
 
   g_return_if_fail(widget);
-  contact = gtk_object_get_data( GTK_OBJECT( widget ), "contact" );
+  contact = g_object_get_data (G_OBJECT (widget ), "contact" );
   
   if( GTK_IS_CLIST( widget ) ) 
   {
@@ -84,7 +88,6 @@ static void contact_list_dnd_drop(GtkWidget *widget,
   }
   else if ( contact != NULL ) 
   {
-      select_clist_row_from_contact( contact );
   }
   else
   {
@@ -103,14 +106,13 @@ static void contact_list_dnd_drop(GtkWidget *widget,
           copy [ selection_data->length ] = 0;
 
           /* Lets see if its a file list, or a URL */
-          if ( (strncmp(copy, "file:", 5) == 0) &&
-               ( MainData->lb_userwin == MainData->lb_userwin_online )) 
+          if (strncmp(copy, "file:", 5) == 0)
           {
-              uris = gnome_uri_list_extract_uris (copy);
+              uris = gnome_vfs_uri_list_parse (copy);
               
               request_file(widget, uris, contact);
-              g_free (copy);    
-              gnome_uri_list_free_strings(uris);
+              g_free (copy);  
+              gnome_vfs_uri_list_free(uris);
           } else if ( (strncmp(copy, "http", 4) == 0) ||
                       (strncmp(copy, "ftp", 3) == 0) ) 
           {
@@ -118,5 +120,7 @@ static void contact_list_dnd_drop(GtkWidget *widget,
           }
       }
       break;
+	 default:
+		g_warning("Unknown info for drag and drop: %d\n", info);
     }
 }
