@@ -1,8 +1,24 @@
-/**************************)
- Main source file
- (c) 1999 Jeremy Wise
- GnomeICU
-***************************/
+/* GnomeICU
+ * Copyright (C) 1998-2004 Jeremy Wise, Olivier Crete
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
+/*
+ * GnomeICU starts here
+ */
 
 #include "common.h"
 
@@ -15,8 +31,6 @@
 #include "gnomeicu-spinner.h"
 #include "groups.h"
 #include "gtkfunc.h"
-/* GNOME2 FIXME : commented out the use of gtkspell */
-/* #include "gtkspell.h" */
 #include "icons.h"
 #include "newsignup.h"
 #include "showlist.h"
@@ -51,6 +65,9 @@
 #include <unistd.h>
 #include <fcntl.h>
 
+#ifdef WITH_GTKSPELL
+# include <gtkspell/gtkspell.h>
+#endif
 
 DWORD our_ip = 0x7f000001; /* localhost */
 DWORD our_port = 0; /* the port to make tcp connections on */
@@ -65,7 +82,6 @@ gchar *Away_Message = NULL;
 gboolean search_in_progress = FALSE;
 
 GtkWidget *status_button = NULL;
-GtkWidget *status_im = NULL;
 
 gchar *configfilename = NULL;
 
@@ -77,16 +93,13 @@ _programs *programs;
 gboolean is_new_user = FALSE;
 gboolean connection_alive = TRUE;
 
-//gint WindowWidth = 160, WindowHeight = 310; GCONF_PREFS
-//gint WindowX = 50, WindowY = 50; GCONF_PREFS
-
 _MainData *MainData;
 
 GtkWidget *app;
 
 static char *geometry;
 
-int preset_status = STATUS_OFFLINE;
+int preset_status = STATUS_ONLINE;
 
 gboolean enable_online_events = FALSE;
 
@@ -98,27 +111,17 @@ guint32 list_time_stamp;
 guint16 record_cnt;
 GSList *Groups = NULL;
 
-/*** Local function declarations ***/
 
 static gint pinger(gpointer data);
-
-void source_drag_data_get (GtkWidget *widget, GdkDragContext *context,
-                           GtkSelectionData *selection_data, guint info,
-                           guint time, gpointer data);
-/* GCONF_PREFS
-void window_save_size (GtkWidget *widget, GdkEventConfigure *event,
-                       gpointer user_data);
-*/
-gint tree_sort_compare (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
-			gpointer user_data);
-                                                                                        
 static void child_died (int);
-
 static void app_init (void);
-
 static gboolean is_already_running(void);
+static void source_drag_data_get (GtkWidget *widget, GdkDragContext *context,
+                                  GtkSelectionData *selection_data, guint info,
+                                  guint time, gpointer data);
+static void gtkspell_change_status(void);
+
 
-/*** Global functions ***/
 void create_tcp_line( void )
 {
 	gint cx;
@@ -180,61 +183,50 @@ void create_tcp_line( void )
 		                          min_tcp_port, max_tcp_port ) );
 }
 
-void ready_set( gboolean flash )
+void
+gnomeicu_set_status_button (gboolean flash)
 {
-	gchar *sts;
-	GtkWidget *label;
-	GtkWidget *pb;
+  static GtkWidget *status_im = NULL;
+  GtkWidget *pb;
+  GtkWidget *label;
 
-#ifdef TRACE_FUNCTION
-	g_print( "ready_set(%08x)\n", Current_Status );
-#endif
+  if (status_im != NULL)
+    gtk_container_remove (GTK_CONTAINER (status_button), status_im);
 
-	if( status_im != NULL )
-		gtk_container_remove( GTK_CONTAINER( status_button ), status_im );
+  status_im = gtk_hbox_new (FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (status_button), status_im);
 
-	status_im = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (status_im);
+  /* set pixbuf */
+  pb = gtk_image_new_from_pixbuf (get_pixbuf_for_status (Current_Status));
+  gtk_box_pack_start (GTK_BOX (status_im), pb, FALSE, FALSE, 0);
 
-	pb = gtk_image_new_from_pixbuf (get_pixbuf_for_status (Current_Status));
-	gtk_widget_show (pb);
-	gtk_box_pack_start (GTK_BOX (status_im), pb, FALSE, FALSE, 0);
+  /* set label */
+  label = gtk_label_new (get_status_str (Current_Status));
+  gtk_box_pack_start (GTK_BOX (status_im), label, TRUE, TRUE, 5);
 
-	sts = g_strconcat( "  ", get_status_str( Current_Status ), NULL );
-	/* If not connected, now it prints "Unknown", but should be */
-	/*	sts = _("Not Connected");	*/
+  gtk_widget_show_all (status_im);
 
-	label = gtk_label_new ( sts );
-	g_free( sts );
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (status_im), label, TRUE, TRUE, 0);
-	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-
-	gtk_container_add( GTK_CONTAINER( status_button ), status_im );
-	gtk_widget_show( status_im );
-
-	tray_update (flash);
+  tray_update (flash);
 }
 
-void init( void )
+void
+gnomeicu_done_login (void)
 {
-	static int flash_timeout_id = 0;
-	static gint pinger_id = 0;
+  static int flash_timeout_id = 0;
+  static gint pinger_id = 0;
 	
-#ifdef TRACE_FUNCTION
-	g_print( "init\n" );
-#endif
-	Done_Login = TRUE;
+  Done_Login = TRUE;
+  connection_alive = TRUE;
 
-	connection_alive = TRUE;
-	if(!pinger_id)
-		pinger_id = g_timeout_add(60000, (GtkFunction)pinger, NULL);
-	
-	if (!flash_timeout_id)
-		flash_timeout_id = g_timeout_add (500, (GtkFunction) flash_messages, NULL );
+  /* setup timers */
 
-        auto_respond_init ();
+  if(!pinger_id)
+    pinger_id = g_timeout_add(60000, (GtkFunction)pinger, NULL);
+
+  if (!flash_timeout_id)
+    flash_timeout_id = g_timeout_add (500, (GtkFunction) flash_messages, NULL );
+
+  auto_respond_init ();
 }
 
 /*** Local functions ***/
@@ -353,59 +345,67 @@ void child_died (int sig_num)
   waitpid (-1, NULL, WNOHANG | WUNTRACED);
 }
 
-static gint app_save_state_cb( GnomeClient *client, gint phase, GnomeRestartStyle save_style, gint shutdown, GnomeInteractStyle interact_style, gint fast, gpointer client_data )
+static gint
+app_save_state_cb (GnomeClient *client, gint phase, GnomeRestartStyle save_style,
+                   gint shutdown, GnomeInteractStyle interact_style, gint fast,
+                   gpointer client_data)
 {
-	gchar *argv[] = {client_data, "-s", NULL, "-g", NULL, NULL};
-	int x,y,w,h;
+  gchar *argv[] = { client_data, "-s", NULL, "-g", NULL, NULL };
+  gint x, y;
 
-#ifdef TRACE_FUNCTION
-	g_print ("app_save_state_cb\n");
-#endif
-
-
-	/* Set commands to clone and restart this application.  Note that we
-	   use the same values for both -- the session management code will
-	   automatically add whatever magic option is required to set the
-	   session id on startup.  */
+  /* Set commands to clone and restart this application.  Note that we
+     use the same values for both -- the session management code will
+     automatically add whatever magic option is required to set the
+     session id on startup.  */
 	
-	switch (Current_Status & 0xffff)
-	  {
-	  case STATUS_ONLINE:
-	    argv[2] = "online";
-	    break;
-	  case STATUS_INVISIBLE:
-	    argv[2] = "invisible";
-	    break;
-	  case STATUS_NA:
-	    argv[2] = "na";
-	    break;
-	  case STATUS_FREE_CHAT:
-	    argv[2] = "freechat";
-	    break;
-	  case STATUS_OCCUPIED:
-	    argv[2] = "occupied";
-	    break;
-	  case STATUS_AWAY:
-	    argv[2] = "away";
-	    break;
-	  case STATUS_DND:
-	    argv[2] = "dnd";
-	    break;
-	  case STATUS_OFFLINE:
-	  default:
-	    argv[2] = "offline";
-	  }
+  switch (Current_Status & 0xffff) {
+  case STATUS_ONLINE:
+    argv[2] = "online";
+    break;
+  case STATUS_INVISIBLE:
+    argv[2] = "invisible";
+    break;
+  case STATUS_NA:
+    argv[2] = "na";
+    break;
+  case STATUS_FREE_CHAT:
+    argv[2] = "freechat";
+    break;
+  case STATUS_OCCUPIED:
+    argv[2] = "occupied";
+    break;
+  case STATUS_AWAY:
+    argv[2] = "away";
+    break;
+  case STATUS_DND:
+    argv[2] = "dnd";
+    break;
+  case STATUS_OFFLINE:
+  default:
+    argv[2] = "offline";
+  }
 
-	gtk_window_get_size( GTK_WINDOW (MainData->window), &w, &h);
-	gtk_window_get_position( GTK_WINDOW (MainData->window), &x, &y);
+  if (MainData->window != NULL) {
+    gint window_w, window_h;
 
-	argv[4] = g_strdup_printf("%dx%d+%d+%d",w,h,x,y);
+    gdk_drawable_get_size(GTK_WIDGET (MainData->window)->window,
+                          &window_w, &window_h);
+    preferences_set_int (PREFS_GNOMEICU_WINDOW_WIDTH, window_w);
+    preferences_set_int (PREFS_GNOMEICU_WINDOW_HEIGHT, window_h);
 
-	gnome_client_set_restart_command (client, 5, argv);
+  }
 
-	g_free(argv[4]);
+  if (MainData->hidden) {
+    x = MainData->x;
+    y = MainData->y;
+  } else
+    gtk_window_get_position (GTK_WINDOW (MainData->window), &x, &y);
 
-	return TRUE;
+  argv[4] = g_strdup_printf ("+%d+%d", x ,y);
+  gnome_client_set_restart_command (client, 5, argv);
+  g_free (argv[4]);
+
+  return TRUE;
 }
 
 void app_init (void)
@@ -413,9 +413,6 @@ void app_init (void)
 	GtkWidget *hbox;
 	BonoboDock* dock;
 	BonoboDockItem* dockitem;
-	GtkCellRenderer *icon_renderer;
-	GtkCellRenderer *nick_renderer;
-	GtkTreeViewColumn *column;
         gint window_w, window_h;
 
 	MainData->xml = gicu_util_open_glade_xml ("main.glade", "app");
@@ -446,78 +443,8 @@ void app_init (void)
 	gtk_widget_show(glade_xml_get_widget (MainData->xml, "main_vbox"));
 	gnome_app_set_contents(GNOME_APP(app), glade_xml_get_widget (MainData->xml, "main_vbox"));
 
-	MainData->contacts_store = gtk_tree_store_new (N_COLUMNS,
-	                                               GDK_TYPE_PIXBUF,
-	                                               G_TYPE_STRING,
-	                                               G_TYPE_INT,
-	                                               G_TYPE_STRING,
-	                                               G_TYPE_INT,
-	                                               G_TYPE_POINTER);
-
-	MainData->visible_store = gtk_tree_store_new (N_COLUMNS,
-	                                               GDK_TYPE_PIXBUF,
-	                                               G_TYPE_STRING,
-	                                               G_TYPE_INT,
-	                                               G_TYPE_STRING,
-	                                               G_TYPE_INT,
-	                                               G_TYPE_POINTER);
-
-	MainData->contacts_tree = glade_xml_get_widget (MainData->xml, "maintreeview");
-	gtk_tree_view_set_model (GTK_TREE_VIEW (MainData->contacts_tree), GTK_TREE_MODEL(MainData->visible_store));
-
-	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (MainData->visible_store),
-					 NICKNAME_COLUMN,
-					 tree_sort_compare,
-					 NULL,
-					 NULL);
-	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(MainData->visible_store),
-					     NICKNAME_COLUMN,
-					     GTK_SORT_ASCENDING);
-
-
-	icon_renderer = gtk_cell_renderer_pixbuf_new();
-	nick_renderer = gtk_cell_renderer_text_new();
-
-	column = gtk_tree_view_column_new();
-	gtk_tree_view_append_column (GTK_TREE_VIEW(MainData->contacts_tree), column);
-
-	gtk_tree_view_column_pack_start (GTK_TREE_VIEW_COLUMN(column), icon_renderer, FALSE);
-	gtk_tree_view_column_pack_start (GTK_TREE_VIEW_COLUMN(column), nick_renderer, TRUE);
-	
-
-	gtk_tree_view_column_set_attributes (GTK_TREE_VIEW_COLUMN(column), nick_renderer,
-	                                     "markup", NICKNAME_COLUMN, NULL);
-	gtk_tree_view_column_set_attributes (GTK_TREE_VIEW_COLUMN(column), icon_renderer,
-	                                     "pixbuf", ICON_COLUMN, NULL);
-
-	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (MainData->contacts_tree), column);
-	
-	MainData->all_tree = glade_xml_get_widget (MainData->xml, "alltreeview");
-	gtk_tree_view_set_model (GTK_TREE_VIEW (MainData->all_tree), GTK_TREE_MODEL (MainData->contacts_store));
-
-	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (MainData->contacts_store),
-					 NICKNAME_COLUMN,
-					 tree_sort_compare,
-					 NULL, NULL);
-	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(MainData->contacts_store),
-					     NICKNAME_COLUMN,
-					     GTK_SORT_ASCENDING);
-
-	icon_renderer = gtk_cell_renderer_pixbuf_new();
-	nick_renderer = gtk_cell_renderer_text_new();
-
-	column = gtk_tree_view_column_new();
-	gtk_tree_view_append_column (GTK_TREE_VIEW(MainData->all_tree), column);
-
-	gtk_tree_view_column_pack_start (GTK_TREE_VIEW_COLUMN(column), icon_renderer, FALSE);
-	gtk_tree_view_column_pack_start (GTK_TREE_VIEW_COLUMN(column), nick_renderer, TRUE);
-
-	gtk_tree_view_column_set_attributes (GTK_TREE_VIEW_COLUMN(column), nick_renderer,
-	                                     "markup", NICKNAME_COLUMN, NULL);
-	gtk_tree_view_column_set_attributes (GTK_TREE_VIEW_COLUMN(column), icon_renderer,
-	                                     "pixbuf", ICON_COLUMN, NULL);
-
-	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (MainData->all_tree), column);
+        /* setup tree view attributes */
+        gnomeicu_tree_create ();
 
 	MainData->window = app;
 	MainData->notebook = glade_xml_get_widget (MainData->xml, "notebook");
@@ -541,16 +468,23 @@ void app_init (void)
 	g_signal_connect (G_OBJECT (status_button), "clicked",
 	                  G_CALLBACK (popup_status_menu), NULL);
 
+
+#ifdef WITH_GTKSPELL
+	preferences_watch_key(PREFS_GNOMEICU_SPELLCHECK, 
+			      gtkspell_change_status);
+#endif
+
 	glade_xml_signal_autoconnect (MainData->xml);
 
-	gnomeicu_spinner_init();
+	gnomeicu_spinner_init ();
 
-	ready_set(FALSE);
+	gnomeicu_set_status_button (FALSE);
 }
 
 int main( int argc, char *argv[] )
 {
 	int a,b,c,d;  /* vars for ip parsing */
+	gchar *file;
 
 	GnomeClient *client;
 	static char *cmdline_status, *cmdline_ip;
@@ -571,10 +505,7 @@ int main( int argc, char *argv[] )
 	SOCKSinit(argv[0]);
 #endif
 
-	if (is_already_running()) {
-	  g_warning("GnomeICU is already running: showing window\n");
-	  exit(1);
-	}
+
 
 
 	srand( time( NULL ) );
@@ -595,8 +526,16 @@ int main( int argc, char *argv[] )
 			    GNOME_PARAM_POPT_TABLE, arguments,
 			    GNOME_PARAM_GNOME_DATADIR, GNOMEICU_DATADIR,
 			    GNOME_PARAM_HUMAN_READABLE_NAME, _("GnomeICU"),
+			    GNOME_PARAM_APP_DATADIR, GNOMEICU_DATADIR,
 			    NULL);
 
+
+
+	if (is_already_running()) {
+	  g_warning("GnomeICU is already running: showing window\n");
+	  exit(1);
+	}
+
 	if (cmdline_ip) {
 	  sscanf(cmdline_ip,"%d.%d.%d.%d",&a,&b,&c,&d);
 	  our_ip = ((DWORD)(a)<<24) |
@@ -607,6 +546,7 @@ int main( int argc, char *argv[] )
 	
         gnomeicu_check_data_dir ();
 
+	app_init ();
 	Read_RC_File();
 
 	if ( !our_info->uin )
@@ -634,10 +574,6 @@ int main( int argc, char *argv[] )
 
 	gnome_config_push_prefix( configfilename );
 
-	/* GNOME2 FIXME : removed gtkspell */
-	/* if( toggles->check_spell )
-		if (gtkspell_start(NULL, ispell_cmd) < 0)
-		fprintf(stderr, "Unable to start ispell; spellchecking disabled.\n"); */
 
 	/* These SIGPIPEs are irritating, ignore them */
 	signal(SIGPIPE, SIG_IGN);
@@ -653,11 +589,16 @@ int main( int argc, char *argv[] )
 	                  G_CALLBACK (app_save_state_cb),
 	                  argv[0]);
 
-	gnome_window_icon_set_default_from_file( GNOMEICU_DATADIR "/pixmaps/gnome-gnomeicu.png" );
+	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
+					  "gnome-gnomeicu.png", TRUE, NULL);
+	if (file != NULL) {
+	  gtk_window_set_default_icon_from_file( file, NULL );
+	  g_free (file);
+	}
 
-	app_init ();
 
-	gnomeicu_tree_create();
+        /* populate loaded data to the tree model */
+	gnomeicu_tree_populate_data ();
 
 	gtk_widget_show(app);
 
@@ -666,12 +607,18 @@ int main( int argc, char *argv[] )
 
 	/* disabled file transfer create_tcp_line(); */
 
-        if ( is_new_user)
+        if (is_new_user)
           v7_new_user(passwd);
         else {
+          /* auto login */
           if (preferences_get_bool (PREFS_GNOMEICU_STARTUP_CONNECT) &&
-	      preset_status != STATUS_OFFLINE)
-            v7_new_login_session();
+	      preset_status != STATUS_OFFLINE) {
+	    if(!strcmp(passwd, "")) /* check for empty password */
+	      show_uin_pwd_dialog();
+
+            if(strcmp(passwd, "")) /* check again */
+              v7_new_login_session();
+          }
           /* bootup unix socket for applet communication */
 	  userserver_init();
 	}
@@ -708,8 +655,11 @@ source_drag_data_get  (GtkWidget	*widget, GdkDragContext *context,
 	g_free( str );
 }
 
+/*
+ * Display the About dialog from Help -> About
+ */
 void
-about (GtkWidget *widget, gpointer data)
+gnomeicu_about (GtkWidget *widget, gpointer data)
 {
   static GtkWidget *about = NULL;
   GdkPixbuf *pixbuf = NULL;
@@ -748,11 +698,7 @@ about (GtkWidget *widget, gpointer data)
   }
 
   about = gnome_about_new ("GnomeICU", VERSION,
-                           /* Translators: Please change the (C) to a real
-                            * copyright character if your character set
-                            * allows it (Hint: iso-8859-1 is one of the
-                            * character sets that has this symbol). */
-                           _("(C) 1999-2003 Jeremy Wise, Olivier Crete, Patrick Sung"),
+                           "Copyright \xc2\xa9 1999-2004 Jeremy Wise, Olivier Cr\xc3\xaate, Patrick Sung",
                            _("GnomeICU is a small, fast and functional "
                              "clone of Mirabilis' ICQ program, specifically "
                              "designed for GNOME."),
@@ -797,18 +743,14 @@ void icq_quit( GtkWidget *widget, gpointer data )
 		contact = contact->next;
 	}
 
-	if (MainData->window != NULL && GTK_WIDGET_VISIBLE (MainData->window)) {
-		gdk_drawable_get_size( GTK_WIDGET( MainData->window )->window,
-		                     &window_w, &window_h);
+	if (MainData->window != NULL) {
+		gdk_drawable_get_size (GTK_WIDGET (MainData->window)->window,
+                                       &window_w, &window_h);
                 preferences_set_int (PREFS_GNOMEICU_WINDOW_WIDTH, window_w);
                 preferences_set_int (PREFS_GNOMEICU_WINDOW_HEIGHT, window_h);
-/*		gdk_window_get_position (GTK_WIDGET (MainData->window)->window,
-		                     &WindowX,
-	                             &WindowY);*/
+
 	}
 	g_object_unref (G_OBJECT (MainData->xml));
-	/* GNOME2 FIXME : gtkspell removed */
-        /* gtkspell_stop(); */
 
 	Save_RC();
 
@@ -821,29 +763,6 @@ void icq_quit( GtkWidget *widget, gpointer data )
 	gtk_main_quit();
 }
 
-
-gint tree_sort_compare (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
-			gpointer user_data)
-{
-	gchar *nicka, *nickb;
-	int result;
-
-	gtk_tree_model_get(model, a, NICKNAME_COLUMN, &nicka, -1); 
-	gtk_tree_model_get(model, b, NICKNAME_COLUMN, &nickb, -1);
-
-	if (!strncmp(_("Not in list"), nicka+3, strlen(_("Not in list"))))
-		result = 1;
-	if (!strncmp(_("Not in list"), nickb+3, strlen(_("Not in list"))))
-		result = -1;
-	else
-		result = strcoll(nicka, nickb);
-
-	g_free(nicka);
-	g_free(nickb);
-
-	return result;
-}
-
 void show_help (void)
 {
 	GError *error = NULL;
@@ -904,3 +823,30 @@ gboolean is_already_running(void)
 
   return FALSE;
 }
+
+#ifdef WITH_GTKSPELL
+
+void gtkspell_change_status(void)
+{
+  GSList *contact = Contacts;
+  GtkSpell *gtkspell;
+  
+  if (preferences_get_bool(PREFS_GNOMEICU_SPELLCHECK))
+    while (contact != NULL ) {
+      if (kontakt->msg_dlg_xml) {
+	gtkspell = gtkspell_new_attach(GTK_TEXT_VIEW(glade_xml_get_widget (kontakt->msg_dlg_xml, "input")),
+				       NULL, NULL);
+	gtkspell_recheck_all(gtkspell);
+      }
+	contact = contact->next;
+    }
+  else
+    while (contact != NULL ) {
+      if ( kontakt->msg_dlg_xml && 
+	   (gtkspell = gtkspell_get_from_text_view(GTK_TEXT_VIEW(glade_xml_get_widget (kontakt->msg_dlg_xml, "input")))))
+	gtkspell_detach(gtkspell);
+      contact = contact->next;
+    }
+}
+
+#endif
