@@ -64,7 +64,6 @@ static void icq_url_window(STORED_MESSAGE_PTR message_text, GSList* contact);
 static gboolean search_nick_timeout(GString *search_nick);
 static void search_nick_in_treeview(GtkTreeView *widget, GdkEventKey *ev,
 				    GString *search_nick, guint *timeout_source);
-static void launch_context_menu(GtkTreeView *widget, GdkEventButton *ev);
 
 enum {
         ICQ_RESPONSE_SEND
@@ -93,8 +92,7 @@ static void icq_user_added_you_msgbox(GSList *contact)
 		break;
 	}
 
-	dialog = gtk_message_dialog_new (GTK_WINDOW (MainData->window),
-	                                 GTK_DIALOG_DESTROY_WITH_PARENT,
+	dialog = gtk_message_dialog_new (NULL, 0,
 	                                 GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
 	                                 sender);
 	gtk_widget_show (dialog);
@@ -168,7 +166,7 @@ static void icq_filetransfer_window(STORED_MESSAGE_PTR message_text,
                                               GTK_STOCK_YES, GTK_RESPONSE_YES,
                                               NULL);
 
-	gtk_window_set_wmclass (GTK_WINDOW (window), "File_Offer", "GnomeICU");
+	gtk_window_set_role (GTK_WINDOW (window), "GnomeICU_FileOffer");
 	gtk_dialog_set_default_response (GTK_DIALOG (window), GTK_RESPONSE_YES);
 
 	set_window_icon( window, "gnomeicu-file.png" );
@@ -275,87 +273,10 @@ static void icq_filetransfer_window(STORED_MESSAGE_PTR message_text,
 static void icq_authorization_request_window(STORED_MESSAGE_PTR message_text, 
 					     GSList* contact)
 {
-	GtkWidget *window = NULL;
-	GtkWidget *dialog_vbox1;
-	GtkWidget *label1;
-	GtkWidget *label2;
-	GtkWidget *label3;
-	GtkWidget *hbox1;
-	GtkWidget *timefield;
-	GtkWidget *scrolledwindow1;
-	GtkWidget *text;
-	gchar* sender;
-	struct tm *my_tm;
-	char msgtime[40];
+  g_assert(message_text != NULL && contact != NULL);
+  
+  auth_receive_request (kontakt->uin, kontakt->nick, message_text->message);
 
-	g_assert(message_text != NULL && contact != NULL);
-
-	window = gtk_dialog_new_with_buttons (_("Authorization Request"),
-                                              GTK_WINDOW (MainData->window),
-                                              GTK_DIALOG_DESTROY_WITH_PARENT,
-                                              _("Refuse"), GTK_RESPONSE_NO,
-                                              _("Allow"), GTK_RESPONSE_YES,
-                                              NULL);
-
-	gtk_window_set_wmclass(GTK_WINDOW(window), "Auth_Request", "GnomeICU");
-	gtk_dialog_set_default_response (GTK_DIALOG (window), GTK_RESPONSE_YES);
-	
-	dialog_vbox1 = GTK_DIALOG (window)->vbox;
-	
-	sender = g_strdup_printf(_("Auth Request from %s:\n"), kontakt->nick);
-	
-	label1 = gtk_label_new ( sender );
-	g_free( sender );
-	gtk_widget_show (label1);
-	gtk_box_pack_start (GTK_BOX (dialog_vbox1), label1, FALSE, FALSE, 0);
-	
-	hbox1 = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox1);
-	gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox1, TRUE, TRUE, 0);
-	
-	label3 = gtk_label_new (_("Time/Date:"));
-	gtk_widget_show (label3);
-	gtk_box_pack_start (GTK_BOX (hbox1), label3, TRUE, TRUE, 0);
-	gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_LEFT);
-	gtk_misc_set_alignment (GTK_MISC (label3), 1, 0.5);
-	
-	timefield = gtk_entry_new ();
-	gtk_widget_show (timefield);
-	gtk_entry_set_editable( GTK_ENTRY( timefield ), FALSE );
-	gtk_box_pack_start (GTK_BOX (hbox1), timefield, FALSE, TRUE, 0);
-	
-	my_tm = localtime (&message_text->time);
-	strftime (msgtime, sizeof (msgtime), _("%c"), my_tm);
-	gtk_entry_set_text( GTK_ENTRY( timefield ), msgtime );
-	
-	label2 = gtk_label_new (_("Reason:"));
-	gtk_widget_show (label2);
-	gtk_box_pack_start (GTK_BOX (dialog_vbox1), label2, FALSE, FALSE, 0);
-	gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);
-	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
-	
-	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
-	gtk_widget_show (scrolledwindow1);
-	gtk_box_pack_start (GTK_BOX (dialog_vbox1), scrolledwindow1, TRUE, 
-			    TRUE, 0);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolledwindow1),
-				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	
-	text = gtk_text_view_new ();
-	gtk_widget_show (text);
-	gtk_text_view_set_wrap_mode( GTK_TEXT_VIEW( text ), 
-				     GTK_WRAP_WORD);
-	gtk_text_view_set_editable( GTK_TEXT_VIEW( text ), FALSE );
-	
-	gtk_text_buffer_insert_at_cursor( gtk_text_view_get_buffer(GTK_TEXT_VIEW( text )), message_text->message, -1 );
-	
-	gtk_container_add (GTK_CONTAINER (scrolledwindow1), text);
-	
-	g_signal_connect (G_OBJECT (window), "response",
-	                  G_CALLBACK (icq_auth_request_response),
-	                  kontakt->uin);
-	
-	gtk_widget_show (window);
 }
 
 
@@ -381,7 +302,7 @@ static void icq_url_window(STORED_MESSAGE_PTR message_text, GSList* contact)
                                               GTK_DIALOG_DESTROY_WITH_PARENT,
                                               GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                               NULL);
-	gtk_window_set_wmclass (GTK_WINDOW (window), "URL", "GnomeICU");
+	gtk_window_set_role (GTK_WINDOW (window), "GnomeICU_URL");
 	gtk_window_set_default_size( GTK_WINDOW( window ), 500, 150 );
 	gtk_dialog_set_default_response (GTK_DIALOG (window), GTK_RESPONSE_CLOSE);
 	
@@ -449,6 +370,8 @@ static void icq_url_window(STORED_MESSAGE_PTR message_text, GSList* contact)
 	g_signal_connect (G_OBJECT (window), "response",
 	                  G_CALLBACK (gtk_widget_destroy), NULL);
 	gtk_widget_show (window);
+
+	g_free(message_text->data);
 }
 
 
@@ -471,7 +394,7 @@ void icq_msgbox( STORED_MESSAGE_PTR message_text, UIN_T uin )
 	switch(message_text->type){
 	case MESSAGE_TEXT:
 		/*** User has received a TEXT message ***/
-	        open_message_dlg_with_message (NULL, uin, message_text);
+	        open_message_dlg_with_message (kontakt, message_text);
 		break;
 	case MESSAGE_CONT_LIST:
 		/*** User has received a CONTACT LIST ***/
@@ -670,7 +593,7 @@ void send_url_window( GtkWidget *widget, char *defaulturl )
                                               GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
                                               NULL);
 	gtk_window_set_default_size (GTK_WINDOW (window), 500, -1);
-	gtk_window_set_wmclass (GTK_WINDOW (window), "SendURL", "GnomeICU");
+	gtk_window_set_role (GTK_WINDOW (window), "GnomeICU_SendURL");
 
 	set_window_icon( window, "gnomeicu-url.png" );
 
@@ -834,7 +757,11 @@ void icq_set_status_online( GtkWidget *widget, gpointer data )
 
   		preset_status = STATUS_ONLINE; 
   		Done_Login = FALSE; 
-                v7_new_login_session();
+
+		if(!strcmp(passwd, "")) 
+		  show_uin_pwd_dialog();
+		if(strcmp(passwd, "")) 
+		  v7_new_login_session();
                 
 	}
 	else
@@ -879,7 +806,7 @@ void icq_set_status_offline( GtkWidget *widget, gpointer data )
 
         enable_online_events = FALSE;
 
-	ready_set(FALSE);
+	gnomeicu_set_status_button (FALSE);
 }
 
 int icq_set_status_away( GtkWidget *widget, gpointer noask )
@@ -893,14 +820,18 @@ int icq_set_status_away( GtkWidget *widget, gpointer noask )
 /*  		create_tcp_line(); */
                 preset_status = STATUS_AWAY;
   		Done_Login = FALSE; 
-                v7_new_login_session();  
+
+		if(!strcmp(passwd, "")) 
+		  show_uin_pwd_dialog();
+		else
+		  v7_new_login_session();  
 	}
 	else
 	{
 		v7_setstatus(mainconnection,  STATUS_AWAY );
-		if(!noask)
-                  auto_respond_pick_msg_window_open ();
 	}
+	if(!noask)
+	  auto_respond_pick_msg_window_open ();
 
 	return FALSE;
 }
@@ -916,14 +847,18 @@ void icq_set_status_na( GtkWidget *widget, gpointer noask )
 /*  		create_tcp_line(); */
 		preset_status = STATUS_NA;
                 Done_Login = FALSE; 
-                v7_new_login_session();  
+
+		if(!strcmp(passwd, "")) 
+		  show_uin_pwd_dialog();		
+		else
+		  v7_new_login_session();  
 	}
 	else
 	{
 		v7_setstatus(mainconnection,  STATUS_NA );
-		if(!noask)
-                  auto_respond_pick_msg_window_open ();
 	}
+	if(!noask)
+	  auto_respond_pick_msg_window_open ();
 }
 
 void icq_set_status_invisible( GtkWidget *widget, gpointer data )
@@ -937,7 +872,11 @@ void icq_set_status_invisible( GtkWidget *widget, gpointer data )
 /*  		create_tcp_line(); */
 		preset_status = STATUS_INVISIBLE;
                 Done_Login = FALSE; 
-                v7_new_login_session();   
+
+		if(!strcmp(passwd, "")) 
+		  show_uin_pwd_dialog();
+		else
+		  v7_new_login_session();   
 	}
 	else
 	{
@@ -956,7 +895,11 @@ void icq_set_status_ffc( GtkWidget *widget, gpointer data )
 /*  		create_tcp_line(); */
 		preset_status = STATUS_FREE_CHAT;
                 Done_Login = FALSE; 
-                v7_new_login_session();  
+
+		if(!strcmp(passwd, "")) 
+		  show_uin_pwd_dialog();	       
+		else
+		  v7_new_login_session();  
 	}
 	else
 	{
@@ -978,14 +921,18 @@ void icq_set_status_occ( GtkWidget *widget, gpointer noask )
 /*  		create_tcp_line(); */
 		preset_status = STATUS_OCCUPIED;
                 Done_Login = FALSE;
-                v7_new_login_session();  
+
+		if(!strcmp(passwd, "")) 
+		  show_uin_pwd_dialog();
+		else
+		  v7_new_login_session();  
 	}
 	else
 	{
 		v7_setstatus(mainconnection,  STATUS_OCCUPIED );
+	}
 		if(!noask)
                   auto_respond_pick_msg_window_open ();
-	}
 }
 
 void icq_set_status_dnd( GtkWidget *widget, gpointer noask )
@@ -1002,14 +949,18 @@ void icq_set_status_dnd( GtkWidget *widget, gpointer noask )
 		/*  create_tcp_line(); */
   		preset_status = STATUS_DND; 
   		Done_Login = FALSE; 
-                v7_new_login_session();  
+
+		if(!strcmp(passwd, "")) 
+		  show_uin_pwd_dialog();
+		else
+		  v7_new_login_session();  
 	}
 	else
 	{
 		v7_setstatus(mainconnection,  STATUS_DND );
-		if(!noask)
-                  auto_respond_pick_msg_window_open ();
 	}
+	if(!noask)
+	  auto_respond_pick_msg_window_open ();
 }
 
 void display_url( GtkWidget *widget, const char *url )
@@ -1156,6 +1107,7 @@ void icq_filetransfer_response (GtkDialog *dialog, int response, gpointer xfer)
 	g_print ("icq_filetransfer_response\n");
 #endif
 
+	/* FIXME: WE NEED TO REFUSE IN A V7 WAY TOO */
 	if (response == GTK_RESPONSE_YES)
 		v7_acceptfile (xfer);
 	else if (response == GTK_RESPONSE_NO)
@@ -1215,15 +1167,6 @@ void icq_send_url_response (GtkDialog *dialog, int response, struct URLInfo *url
 	g_free(urlinfo);
 }
 
-/* FIXME: Come on.. clean this up, jwise.. this is a pathetic hack. */
-void grab_history( GtkWidget *widget, gpointer data )
-{
-	GSList *contact;
-/*	GSList *contact = gtk_object_get_data( GTK_OBJECT( gtk_menu_get_active( gtk_object_get_data( GTK_OBJECT( widget ), "menu" ) ) ), "contact" );*/
-	contact = gtk_object_get_data (GTK_OBJECT (widget), "contact");
-	history_display( NULL, contact->data );
-}
-
 void request_file_action ( GSList *completed_files )
 {
     GtkWidget *dialog, *dialog_vbox, *main_vbox, *label;
@@ -1415,244 +1358,6 @@ gchar *get_foreground_for_status( gint status)
 	return str;
 }
 
-gint contacts_tree_button_cb(GtkTreeView *widget, GdkEventButton *ev, gpointer data)
-{
-	if (ev->type == GDK_BUTTON_PRESS && ev->button == 3) {
-		launch_context_menu(widget, ev);
-	}
-
-	return FALSE;
-}
-
-static gboolean search_nick_timeout(GString *search_nick)
-{
-	/* search_nick can't be NULL here because it is created before we add
-	   the first timeout */
-	g_string_erase(search_nick, 0, -1);
-	return FALSE;
-}
-
-static void search_nick_in_treeview(GtkTreeView *widget, GdkEventKey *ev,
-				    GString *search_nick, guint *timeout_source)
-{
-	GtkTreeModel *tree_model;
-	GtkTreeIter iter, child;
-	GtkTreePath *tree_path;
-	gboolean valid;
-	Contact_Member *contact;
-	gchar *nick_collated, *search_collated;
-	gboolean child_valid = FALSE;
-
-	/* we cancel the current timeout if it exists */
-	if (*timeout_source) {
-		g_source_remove(*timeout_source);
-		*timeout_source = 0;
-	}
-
-	g_string_append_unichar(search_nick, gdk_keyval_to_unicode(ev->keyval));
-	search_collated = g_utf8_collate_key(search_nick->str, search_nick->len);
-
-	tree_model = gtk_tree_view_get_model(widget);
-
-	gtk_tree_view_get_cursor(widget, &tree_path, NULL);
-	if (tree_path == NULL) {
-		/* there is no selection => we take the first thing */
-		valid = gtk_tree_model_get_iter_first(tree_model, &iter);
-		child_valid = FALSE;
-	} else {
-		/* there's already a selection => we'll start there */
-		gtk_tree_model_get_iter(tree_model, &child, tree_path);
-		if (!gtk_tree_model_iter_parent(tree_model, &iter, &child)) {
-			/* child has no parent
-			   => it's a group, and not a contact */
-			iter = child;
-			child_valid = FALSE;
-		} else {
-			/* we have the child where we'll start */
-			child_valid = TRUE;
-		}
-		/* we have a current iter, so we know it's valid */
-		valid = TRUE;
-	}
-	gtk_tree_path_free(tree_path);
-
-	/* now we loop on the treeview to find the first nickname that
-	   could match */
-	while (valid) {
-		if (child_valid) {
-			/* we already have a valid child, so we don't
-			   start at the beginning */
-			child_valid = FALSE;
-			valid = TRUE;
-		} else {
-			valid = gtk_tree_model_iter_children(tree_model, &child, &iter);
-		}
-		while (valid) {
-			gtk_tree_model_get(tree_model, &child, CONTACT_COLUMN, &contact, -1);
-
-			nick_collated = g_utf8_collate_key(contact->nick, -1);
-			if (!g_ascii_strncasecmp(nick_collated, search_collated, search_nick->len)) {
-				/* this nickname matches the search
-				   => we select the contact */
-#if GTK_CHECK_VERSION(2,2,0)
-				tree_path = gtk_tree_model_get_path(tree_model, &child);
-				gtk_tree_view_expand_to_path(widget, tree_path);
-#else
-				/* gtk_tree_view_expand_to_path is a GTK+ 2.2
-				 * API addition, so we have to do the same thing
-				 * manually for GTK+ 2.0 */
-				tree_path = gtk_tree_model_get_path(tree_model, &iter);
-				gtk_tree_view_expand_row(widget, tree_path, TRUE);
-				gtk_tree_path_free(tree_path);
-				tree_path = gtk_tree_model_get_path(tree_model, &child);
-#endif
-				gtk_tree_view_set_cursor(widget, tree_path, NULL, FALSE);
-				gtk_tree_path_free(tree_path);
-				g_free(nick_collated);
-				break;
-			}
-
-			g_free(nick_collated);
-			valid = gtk_tree_model_iter_next(tree_model, &child);
-		}
-
-		if (valid) /* we have found someone => stop */
-			break;
-		valid = gtk_tree_model_iter_next(tree_model, &iter);
-	}
-
-	g_free(search_collated);
-
-	/* this will erase search_nick 1 second later if no other
-	   useful key is pressed */
-	*timeout_source = g_timeout_add(1000, (GSourceFunc) search_nick_timeout, search_nick);
-}
-
-gint contacts_tree_key_cb(GtkTreeView *widget, GdkEventKey *ev, gpointer data)
-{
-	static GString *search_nick = NULL;
-	static guint timeout_source = 0;
-
-	if (ev->type == GDK_KEY_PRESS && ev->keyval == GDK_Return) {
-		contacts_tree_activated_cb(widget, NULL, NULL, NULL);
-	} else if (ev->type == GDK_KEY_PRESS && gdk_keyval_to_unicode(ev->keyval) != 0) {
-		if (search_nick == NULL) {
-			search_nick = g_string_sized_new(12);
-		}
-		/* we search the nickname in the treeview */
-		search_nick_in_treeview(widget, ev, search_nick, &timeout_source);
-	} else if (ev->type == GDK_KEY_PRESS) {
-		/* the key is not a valid character
-		   => we'll start a new search next time */
-		if (search_nick != NULL) {
-			g_string_erase(search_nick, 0, -1);
-		}
-
-		/* we cancel the current timeout if it exists */
-		if (timeout_source) {
-			g_source_remove(timeout_source);
-			timeout_source = 0;
-		}
-	}
-
-	return FALSE;
-}
-
-void contacts_tree_activated_cb(GtkTreeView *treeview, GtkTreePath *arg1,
-                                GtkTreeViewColumn *arg2, gpointer user_data)
-{
-	GtkTreeSelection *sel;
-	GtkTreeIter iter;
-	GtkTreeModel *model;
-	gchar *uid;
-	Contact_Member *contact;
-	GSList *lcontact;
-
-	sel = gtk_tree_view_get_selection (treeview);
-	g_assert (sel != NULL);
-	if (gtk_tree_selection_get_selected (sel, &model, &iter)) {
-		gtk_tree_model_get (model, &iter, UID_COLUMN, &uid,
-		                    CONTACT_COLUMN, &contact, -1);
-
-		if (uid) {
-		        lcontact = Find_User(contact->uin);
-		  
-			if (g_slist_length (contact->stored_messages))
-				show_contact_message (lcontact);
-			else
-			  if (contact->msg_dlg_xml != NULL) {
-			    gtk_widget_show_all(gtk_bin_get_child(GTK_BIN(glade_xml_get_widget( contact->msg_dlg_xml, "message_dialog"))));
-			    gtk_window_present(GTK_WINDOW(glade_xml_get_widget( contact->msg_dlg_xml, "message_dialog")));
-			  }
-			  else
-			    open_message_dlg_with_message (lcontact, 0, NULL);
-		} else {
-		  if (gtk_tree_view_row_expanded (treeview, arg1))
-		    gtk_tree_view_collapse_row(treeview, arg1);
-		  else
-		    gtk_tree_view_expand_row(treeview, arg1, TRUE);
-		}
-	g_free(uid);
-	}
-}
-
-
-gint contacts_tree_popup_cb(GtkTreeView *widget, gpointer data)
-{
-	launch_context_menu(widget, NULL);
-
-	return FALSE;
-}
-
-static void launch_context_menu(GtkTreeView *widget, GdkEventButton *ev)
-{
-	GtkTreeSelection *sel;
-	GtkTreeIter iter;
-	GtkTreeModel *model;
-	gchar *uid;
-	gint gid;
-	Contact_Member *contact;
-	GtkTreePath *path = NULL;
-	GtkTreeViewColumn *col;
-	gboolean cont = FALSE;
-
-	sel = gtk_tree_view_get_selection (widget);
-	cont = gtk_tree_selection_get_selected (sel, &model, &iter);
-
-	if (ev) {
-		gtk_tree_view_get_path_at_pos (widget, ev->x, ev->y, &path,
-		                               &col, NULL, NULL);
-		if (!path)
-			return;
-		cont = gtk_tree_model_get_iter (model, &iter, path);
-	}
-
-	if (cont) {
-		gtk_tree_model_get (model, &iter,
-		                    UID_COLUMN, &uid,
-		                    CONTACT_COLUMN, &contact,
-				    GID_COLUMN, &gid, -1);
-
-		if (uid) {
-			GtkWidget *personal_menu;
-			personal_menu = user_popup (contact);
-			gtk_menu_popup (GTK_MENU (personal_menu), NULL, NULL, NULL,
-			                NULL, ev != NULL ? ev->button : 1,
-					ev != NULL ? ev->time: 0);
-		} else if (gid && strcmp(gtk_widget_get_name(GTK_WIDGET(widget)),"maintreeview")) {
-		  GtkWidget *group_menu;
-		  group_menu = group_popup (gid);
-		  gtk_menu_popup (GTK_MENU (group_menu), NULL, NULL, NULL,
-				  NULL, ev != NULL ? ev->button : 1,
-				  ev != NULL ? ev->time: 0);
-		}
-		g_free(uid);
-	}
-
-	if (path)
-		gtk_tree_path_free (path);
-}
-
 gboolean is_connected(GtkWindow *parent_window, const gchar *message)
 {
 	if (!mainconnection || Current_Status == STATUS_OFFLINE) {
