@@ -34,14 +34,8 @@ gint hide_dont_kill( GtkWidget *widget );
 gint hide_ch_window( GtkWidget *widget, GdkEvent *event, GtkWidget *window );
 int popup_status_menu( GtkWidget *widget,  gpointer data );
 gint ctrl_enter_cb( GtkWidget *widget, GdkEventKey *ev, int sock );
-void grab_history( GtkWidget *widget, gpointer data );
 void request_file_action( GSList *completed_files );
 gchar *get_foreground_for_status(gint status);
-gint contacts_tree_button_cb (GtkTreeView *widget, GdkEventButton *ev, gpointer data);
-gint contacts_tree_key_cb (GtkTreeView *widget, GdkEventKey *ev, gpointer data);
-void contacts_tree_activated_cb(GtkTreeView *treeview, GtkTreePath *arg1,
-                                GtkTreeViewColumn *arg2, gpointer user_data);
-gint contacts_tree_popup_cb (GtkTreeView *widget, gpointer data);
 gboolean is_connected(GtkWindow *parent_window, const gchar *message);
 
 #endif /* __GTKFUNC_H__ */
