@@ -52,10 +52,10 @@ extern datum icudb_fetch (DB_FILE file, datum key);
 #define MESG_ISRECV(mesg)  (mesg[0] & 2)
 #define MESG_SETRECV(mesg) (mesg[0] |= 2)
 
-extern DB_FILE icudb_open (gchar *db_path, gint mode);
+extern DB_FILE icudb_open (const char *db_path, int mode);
 extern int icudb_close_all (void);
 extern int icudb_close (DB_FILE file);
-extern int icudb_close_dbpath (gchar *db_path );
+extern int icudb_close_dbpath (const char *db_path);
 #endif
 
 #endif
