@@ -9,6 +9,7 @@
 #include "emoticons.h"
 #include "gnomeicu.h"
 #include "gtkfunc.h"
+#include "history.h"
 #include "icons.h"
 #include "response.h"
 #include "rtf-reader.h"
@@ -22,13 +23,17 @@
 #include <gtk/gtk.h>
 #include <string.h>
 
+#ifdef WITH_GTKSPELL
+# include <gtkspell/gtkspell.h>
+#endif
+
 static void message_cb_menu_position (GtkMenu *menu, gint *x, gint *y, gboolean* push_in, gpointer user_data);
 
 static gboolean msg_url_motion_notify_event (GtkWidget * w, GdkEventMotion * event, gpointer user_data);
 static gboolean msg_url_text_button_pressed (GtkWidget * w, GdkEventButton * event, gpointer unused);
 static const char* find_next_url (const char * pch);
 static char* get_url_from_location (GtkWidget * w, int x, int y);
-static gboolean msg_blink_timeout(GSList *contact);
+static gboolean msg_blink_timeout(Contact_Member *contact);
 
 
 
@@ -97,82 +102,79 @@ init_message_tags() {
 }
     
 
-void gnomeicu_text_buffer_insert_with_emoticons (GtkTextBuffer *text_buffer, gchar *text)
+void
+gnomeicu_text_buffer_insert_with_emoticons (GtkTextBuffer *text_buffer,
+                                            GtkTextMark *mark, const gchar *text)
 {
-    GSList *e;
-    gchar **arr;
-    GtkTextIter end_iter;
-    gboolean found = FALSE;
-    gboolean use_emoticon = TRUE;
-    gchar *theme_name;
-    
-#ifdef TRACE_FUNCTION
-	g_print("gnomeicu_text_buffer_insert_with_emoticons\n");
-#endif
+  GSList *e;
+  gchar **arr;
+  GtkTextIter iter;
+  gboolean found = FALSE;
+  gchar *theme_name;
 
-    g_assert( text_buffer != NULL );
-    g_assert( text  != NULL );
+  g_assert (text_buffer != NULL);
+  g_assert (text  != NULL);
 
-    theme_name = preferences_get_string (PREFS_GNOMEICU_EMOTICON_THEMES);
+  theme_name = preferences_get_string (PREFS_GNOMEICU_EMOTICON_THEMES);
+  if (!theme_name)
+    theme_name = g_strdup ("None");
 
-    if (strcmp (theme_name, "None"))
+  if (strcmp (theme_name, "None"))
+    for (e = Emoticons; e != NULL; e = e->next) {
+      if (g_strrstr (text, ((Emoticon*)e->data)->string)) {
+        found = TRUE;
+        arr = g_strsplit (text, ((Emoticon*)e->data)->string, 2);
+        gnomeicu_text_buffer_insert_with_emoticons (text_buffer, mark, arr[0]);
 
-      for (e = Emoticons; e != NULL; e = e->next) {
-	if (g_strrstr (text, ((Emoticon*)e->data)->string)) {
-	    found = TRUE;
-	    arr = g_strsplit (text, ((Emoticon*)e->data)->string, 2);
-	    gnomeicu_text_buffer_insert_with_emoticons (text_buffer, arr[0]);
-	    
-	    gtk_text_buffer_get_end_iter (text_buffer, &end_iter);
-	    if (use_emoticon)
-		gtk_text_buffer_insert_pixbuf (text_buffer, &end_iter,
-					   ((Emoticon*)e->data)->pixbuf);
-	    else
-	    	gtk_text_buffer_insert (text_buffer, &end_iter,
-	    	                        ((Emoticon*)e->data)->string, -1);
-	    gnomeicu_text_buffer_insert_with_emoticons (text_buffer, arr[1]);
-	    g_strfreev (arr);
-	    break;
-	}
-    }
-
-    g_free (theme_name);
-    
-    if (!found) {
-
-      const char * pch;
-      GtkTextIter mark_start;	
-      GtkTextIter mark_end;
-      GtkTextIter start;
-      GtkTextMark * mark;
-
-      gtk_text_buffer_get_end_iter (text_buffer, &end_iter);
-      mark = gtk_text_buffer_create_mark (text_buffer, "blah", &end_iter, TRUE);
-
-      gtk_text_buffer_insert (text_buffer, &end_iter, text, -1);
-      
-      gtk_text_buffer_get_iter_at_mark (text_buffer, &start, mark);
-      gtk_text_buffer_get_end_iter (text_buffer, &end_iter);
-
-      /* markup URLs only if url tag is present */
-
-      if (gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(text_buffer),
-				    "url")) {
-	pch = text;
-	while ((pch = find_next_url (pch))) {
-	  const char * end = pch + 1;
-	  while (*end && !g_unichar_isspace(g_utf8_get_char(end)))
-	    end = g_utf8_next_char(end);
-	  mark_start = start;
-	  gtk_text_iter_forward_chars (&mark_start, g_utf8_strlen(text,pch-text));
-	  mark_end = mark_start;
-	  gtk_text_iter_forward_chars (&mark_end, g_utf8_strlen(pch,end-pch));
-	  gtk_text_buffer_remove_all_tags (text_buffer, &mark_start, &mark_end);
-	  gtk_text_buffer_apply_tag_by_name (text_buffer, "url", &mark_start, &mark_end);
-	  pch = *end ? end+1 : NULL;
-	}
+        gtk_text_buffer_get_iter_at_mark (text_buffer, &iter, mark);
+	gtk_text_buffer_insert_pixbuf (text_buffer, &iter,
+				       ((Emoticon*)e->data)->pixbuf);
+        gnomeicu_text_buffer_insert_with_emoticons (text_buffer, mark, arr[1]);
+        g_strfreev (arr);
+        break;
       }
     }
+
+  g_free (theme_name);
+
+  if (!found) {
+    const char * pch;
+    GtkTextIter url_start;	
+    GtkTextIter url_end;
+    GtkTextIter start;
+    GtkTextMark *text_mark;
+
+    gtk_text_buffer_get_iter_at_mark (text_buffer, &iter, mark);
+    /* save a mark for beginning of text */
+    text_mark = gtk_text_buffer_create_mark (text_buffer, NULL, &iter, TRUE);
+
+    gtk_text_buffer_insert (text_buffer, &iter, text, -1);
+
+    gtk_text_buffer_get_iter_at_mark (text_buffer, &start, text_mark);
+
+    /* markup URLs only if url tag is present */
+    if (gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(text_buffer),
+                                  "url")) {
+      pch = text;
+      while ((pch = find_next_url (pch))) {
+        const char *end = pch + 1;
+
+        /* find the end loc of the URL */
+        while (*end && !g_unichar_isspace(g_utf8_get_char(end)))
+          end = g_utf8_next_char(end);
+        url_start = start;
+        gtk_text_iter_forward_chars (&url_start, g_utf8_strlen (text, pch-text));
+        url_end = url_start;
+        gtk_text_iter_forward_chars (&url_end, g_utf8_strlen (pch,end-pch));
+        gtk_text_buffer_remove_all_tags (text_buffer, &url_start, &url_end);
+        gtk_text_buffer_apply_tag_by_name (text_buffer, "url",
+                                           &url_start, &url_end);
+        pch = *end ? end+1 : NULL;
+      }
+
+      gtk_text_buffer_delete_mark (text_buffer, text_mark);
+    }
+  }
 }
 
 
@@ -180,6 +182,8 @@ void
 gnomeicu_text_view_insert_text(GtkTextView* text_view, gchar* text)
 {
     GtkTextBuffer* text_buffer;
+    GtkTextIter iter;
+    GtkTextMark *mark;
     gchar* result;
 
 #ifdef TRACE_FUNCTION
@@ -207,7 +211,10 @@ gnomeicu_text_view_insert_text(GtkTextView* text_view, gchar* text)
 	    g_warning("Can't convert! Copying directly\n");
 	    result = g_strdup(text);
 	  }
-	    gnomeicu_text_buffer_insert_with_emoticons (text_buffer, result);
+          gtk_text_buffer_get_end_iter (text_buffer, &iter);
+          mark = gtk_text_buffer_create_mark (text_buffer, NULL, &iter, FALSE);
+          gnomeicu_text_buffer_insert_with_emoticons (text_buffer, mark, result);
+          gtk_text_buffer_delete_mark (text_buffer, mark);
 	}
     else 
 	{
@@ -261,7 +268,7 @@ add_message_to_conversation( GtkTextView* conversation,
     /* Show the time of this message in the time widget */
     my_tm = localtime (time);
     strftime (short_msgtime, sizeof (short_msgtime), _("%X"), my_tm);
-    short_msgtime_utf8 = convert_to_utf8(short_msgtime);
+    short_msgtime_utf8 = g_locale_to_utf8(short_msgtime, -1, NULL, NULL, NULL);
 
     /* Strip the \r's out of the message... */
     stripped_str = strip_cr (text);
@@ -292,7 +299,7 @@ add_message_to_conversation( GtkTextView* conversation,
 				 gtk_text_buffer_get_mark(text_buffer, 
 							  "end_mark"),
 				 0.1,
-				 FALSE, 0, 0);
+				 FALSE, 0, 1);
     
 
     g_free (header);
@@ -301,7 +308,7 @@ add_message_to_conversation( GtkTextView* conversation,
 }
 
 
-void open_message_dlg_with_message (GSList *contact, UIN_T uin,
+void open_message_dlg_with_message (Contact_Member *contact,
                                     STORED_MESSAGE_PTR message_text)
 {
 	GladeXML *xml;
@@ -309,142 +316,164 @@ void open_message_dlg_with_message (GSList *contact, UIN_T uin,
 	GtkWidget *chat_toggle;
 	GtkWidget *conversation;
 	GtkWidget *input;
-	GtkWidget *readnext_button;
 	GtkWidget *history_button;
 	GtkWidget *send_button;
 	GtkWidget *usermenu_button;
 
 	GtkTextBuffer* text_buffer;
 	GtkTextIter end_iter;
-
-#ifdef TRACE_FUNCTION
-	g_print("open_message_dlg_with_message\n");
+	STORED_MESSAGE_PTR msg;
+	GSList *message_temp;
+#ifdef WITH_GTKSPELL
+	GtkSpell *gtkspell;
 #endif
 
-	if (contact == NULL)
-		contact = Find_User (uin);
-
-	if (contact == NULL)
-		return;
-
-	if (kontakt->msg_dlg_xml != NULL) {
-		xml = kontakt->msg_dlg_xml;
-		if (message_text && kontakt->newmsg_blink_timeout == 0 &&
-		    gtk_window_get_focus(GTK_WINDOW(glade_xml_get_widget (xml, "message_dialog"))) &&
-		    !GTK_WIDGET_HAS_FOCUS(gtk_window_get_focus(GTK_WINDOW(glade_xml_get_widget (xml, "message_dialog")))))
-		  kontakt->newmsg_blink_timeout = gtk_timeout_add(500, (GtkFunction) msg_blink_timeout, contact);
+        g_assert (contact != NULL);
 
+	if (contact->msg_dlg_xml != NULL) {
+          xml = contact->msg_dlg_xml;
+          if (message_text && contact->newmsg_blink_timeout == 0 &&
+              gtk_window_get_focus(GTK_WINDOW(glade_xml_get_widget (xml, "message_dialog"))) &&
+              !GTK_WIDGET_HAS_FOCUS(gtk_window_get_focus(GTK_WINDOW(glade_xml_get_widget (xml, "message_dialog")))))
+            contact->newmsg_blink_timeout = gtk_timeout_add(500, (GtkFunction) msg_blink_timeout, contact);
 	} else {
-		xml = gicu_util_open_glade_xml ("message.glade", "message_dialog");
-		if (xml == NULL)
-			return;
+          xml = gicu_util_open_glade_xml ("message.glade", "message_dialog");
+          if (xml == NULL)
+            return;
 
-		kontakt->msg_dlg_xml = xml;
+          contact->msg_dlg_xml = xml;
 
-		chat_toggle = glade_xml_get_widget (xml, "chat_toggle");
-		if (preferences_get_bool (PREFS_GNOMEICU_DEFAULT_CHAT_MODE))
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (chat_toggle), TRUE);
-		conversation = glade_xml_get_widget (xml, "conversation");
+          chat_toggle = glade_xml_get_widget (xml, "chat_toggle");
+          if (preferences_get_bool (PREFS_GNOMEICU_DEFAULT_CHAT_MODE))
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (chat_toggle), TRUE);
+          conversation = glade_xml_get_widget (xml, "conversation");
 
-		g_signal_connect (conversation, "motion_notify_event",
-				  G_CALLBACK(msg_url_motion_notify_event), NULL);
+          g_signal_connect (conversation, "motion_notify_event",
+                            G_CALLBACK(msg_url_motion_notify_event), NULL);
 
-		g_signal_connect (conversation, "button_press_event",
-				  G_CALLBACK(msg_url_text_button_pressed), NULL);
+          g_signal_connect (conversation, "button_press_event",
+                            G_CALLBACK(msg_url_text_button_pressed), NULL);
 
-		gtk_window_set_type_hint (GTK_WINDOW(glade_xml_get_widget (xml, "message_dialog")), GDK_WINDOW_TYPE_HINT_NORMAL);
+          gtk_window_set_type_hint (GTK_WINDOW(glade_xml_get_widget (xml, "message_dialog")), GDK_WINDOW_TYPE_HINT_NORMAL);
 
-		/* Set the tags table to the global one */
-		if ( ! message_tags )
-		    init_message_tags();
-		gtk_text_view_set_buffer(GTK_TEXT_VIEW(conversation), 
-					 gtk_text_buffer_new(message_tags));
+          /* Set the tags table to the global one */
+          if ( ! message_tags )
+            init_message_tags();
+          gtk_text_view_set_buffer(GTK_TEXT_VIEW(conversation), 
+                                   gtk_text_buffer_new(message_tags));
 
-		/* Add a mark at the end of the converstaion */
-		text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(conversation));
-		gtk_text_buffer_get_end_iter(text_buffer, &end_iter);
-		gtk_text_buffer_create_mark(text_buffer, "end_mark",
-					    &end_iter, FALSE);
-		gtk_text_buffer_create_mark(text_buffer, "insert_mark",
-					    &end_iter, TRUE);
+          /* Add a mark at the end of the converstaion */
+          text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(conversation));
+          gtk_text_buffer_get_end_iter(text_buffer, &end_iter);
+          gtk_text_buffer_create_mark(text_buffer, "end_mark",
+                                      &end_iter, FALSE);
+          gtk_text_buffer_create_mark(text_buffer, "insert_mark",
+                                      &end_iter, FALSE);
 
-		glade_xml_signal_autoconnect (xml);
+	  gtk_window_set_role(GTK_WINDOW(glade_xml_get_widget(xml, "message_dialog")), "GnomeICU_Message");
+
+          glade_xml_signal_autoconnect (xml);
 	}
 
 	msg_dlg = glade_xml_get_widget (xml, "message_dialog");
         set_window_icon( msg_dlg, "gnomeicu-message.png");
-	gtk_window_set_title (GTK_WINDOW (msg_dlg), kontakt->nick);
+	gtk_window_set_title (GTK_WINDOW (msg_dlg), contact->nick);
 	chat_toggle = glade_xml_get_widget (xml, "chat_toggle");
 	conversation = glade_xml_get_widget (xml, "conversation");
 	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (chat_toggle))) {
-	    gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(conversation)), "", -1);
+          gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(conversation)), "", -1);
 	}
 	input = glade_xml_get_widget (xml, "input");
 
 	if (!GTK_WIDGET_VISIBLE (input)) {
 	    gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(input)), "", -1);
 	}
-	gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "message_nick")), kontakt->nick);
+	gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "message_nick")), contact->nick);
 	gtk_image_set_from_pixbuf(
 				  GTK_IMAGE(glade_xml_get_widget (xml, "message_status")),
-				  get_pixbuf_for_status(kontakt->status));
+				  get_pixbuf_for_status(contact->status));
 
-	kontakt->read_next = readnext_button = glade_xml_get_widget (xml, "readnext_button");
 	history_button = glade_xml_get_widget (xml, "history_button");
 	send_button = glade_xml_get_widget (xml, "send_button");
 	usermenu_button = glade_xml_get_widget (xml, "usermenu_button");
 
 	g_object_set_data (G_OBJECT (msg_dlg), "contact", contact);
-	g_object_set_data (G_OBJECT (history_button), "contact", contact);
 	g_object_set_data (G_OBJECT (input), "contact", contact);
 	g_object_set_data (G_OBJECT (input), "message_dialog", msg_dlg);
 	g_object_set_data (G_OBJECT (usermenu_button), "contact", contact);
-	/* Turn on spell checking if so desired */
-	/* GNOME2 FIXME : removed gtkspell */
-	/* if (toggles->check_spell)
-	{
-		if (!gtkspell_running())
-			gtkspell_start(NULL, ispell_cmd);
-		gtkspell_attach(GTK_TEXT(input));
-		} */
+	g_signal_connect(G_OBJECT(history_button), "clicked", 
+			 G_CALLBACK (history_display), contact);
+#ifdef WITH_GTKSPELL
+	gtkspell = gtkspell_get_from_text_view(GTK_TEXT_VIEW(input));
+	if (!gtkspell && preferences_get_bool(PREFS_GNOMEICU_SPELLCHECK)) {
+	  gtkspell = gtkspell_new_attach(GTK_TEXT_VIEW(input), NULL, NULL);
+	}
+#endif
 
 	if (message_text != NULL) {
 	    /* Show the time of this message in the time widget */
 	    add_message_to_conversation(GTK_TEXT_VIEW(conversation),
-					kontakt,
+					contact,
 					&message_text->time,
 					message_text->message);
-	} else {
-		GtkTextBuffer* text_buffer;
-		text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(conversation));
-		gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(conversation), 
-			gtk_text_buffer_get_mark(text_buffer, "end_mark"),
-			0.1, FALSE, 0, 0);
-		gtk_widget_set_sensitive (readnext_button, FALSE);
+	} 
+
+	
+	/* show all messages */
+
+	if (contact->stored_messages) {
+	msg_next_restart:
+	  for (message_temp = contact->stored_messages;
+	       message_temp; message_temp = message_temp->next) {
+	    msg = (STORED_MESSAGE_PTR)contact->stored_messages->data;
+	    if (msg->type == MESSAGE_TEXT) {
+	      contact->stored_messages = g_slist_remove (contact->stored_messages, msg );		
+	      add_message_to_conversation(GTK_TEXT_VIEW(conversation),
+					  contact,
+					  &msg->time,
+					  msg->message);
+		
+	      g_free( msg->data );
+	      g_free( msg->message );
+	      g_free( msg );
+	      goto msg_next_restart;
+		
+	    }
+	  }
+	  
+	  if ( !contact->stored_messages) 
+	    gnomeicu_tree_set_contact_icon (contact, get_pixbuf_for_status (contact->status));
+	  
+	  Shortcut_Update (contact);
 	}
 
+	
+
+	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(conversation), 
+				     gtk_text_buffer_get_mark(
+                       gtk_text_view_get_buffer(GTK_TEXT_VIEW(conversation)), 
+		       "end_mark"),
+				     0.1, FALSE, 0, 1);
+	
+
 	gtk_widget_grab_focus (input);
 	gtk_widget_show_all (msg_dlg);
 }
 
-gboolean msg_blink_timeout(GSList *contact) 
+gboolean msg_blink_timeout(Contact_Member *contact) 
 {
-  GtkWidget *msg_dlg = glade_xml_get_widget (kontakt->msg_dlg_xml, 
+  GtkWidget *msg_dlg = glade_xml_get_widget (contact->msg_dlg_xml, 
 					     "message_dialog");
 
-#ifdef TRACE_FUNCTION
-	g_print("msg_blink_timeout\n");
-#endif
-
-  if (! kontakt->newmsg_blink_timeout) {
-    kontakt->newmsg_blink = 0;
+  if (!contact->newmsg_blink_timeout) {
+    contact->newmsg_blink = 0;
     set_window_icon( msg_dlg, "gnomeicu-message.png");
     return FALSE;
   }
   
-  kontakt->newmsg_blink = !kontakt->newmsg_blink;
+  contact->newmsg_blink = !contact->newmsg_blink;
   
-  if (kontakt->newmsg_blink)
+  if (contact->newmsg_blink)
     set_window_icon( msg_dlg, "gnomeicu-blank.png");
   else
     set_window_icon( msg_dlg, "gnomeicu-message.png");
@@ -459,7 +488,7 @@ void send_message (GtkWidget *dlg)
 {
 	GtkWidget *entry;
 	GtkWidget *conversation;
-	GSList *contact;
+	Contact_Member *contact;
 	gchar *buf, *tempstr, *convtext;
 	GtkWidget *toggle;
 
@@ -480,9 +509,9 @@ void send_message (GtkWidget *dlg)
 	if (!is_connected(GTK_WINDOW(dlg), _("You can not send messages while being disconnected.")))
 		return;
 
-	entry = glade_xml_get_widget (kontakt->msg_dlg_xml, "input");
-	toggle = glade_xml_get_widget (kontakt->msg_dlg_xml, "chat_toggle");
-	conversation = glade_xml_get_widget (kontakt->msg_dlg_xml, "conversation");
+	entry = glade_xml_get_widget (contact->msg_dlg_xml, "input");
+	toggle = glade_xml_get_widget (contact->msg_dlg_xml, "chat_toggle");
+	conversation = glade_xml_get_widget (contact->msg_dlg_xml, "conversation");
 
 	if (toggle == NULL || entry == NULL || dlg == NULL || conversation == NULL)
 		return;
@@ -503,13 +532,13 @@ void send_message (GtkWidget *dlg)
 	convtext = add_cr(tempstr);
 	g_free(tempstr);
 	
-	if (kontakt->status == STATUS_OFFLINE && strlen(convtext) > 450) {
+	if (contact->status == STATUS_OFFLINE && strlen(convtext) > 450) {
 	  GtkWidget *dialog;
 	  GtkWidget *button;
 	  
 	  g_free(convtext);
 
-	  dialog = gtk_message_dialog_new (GTK_WINDOW (glade_xml_get_widget (kontakt->msg_dlg_xml, "message_dialog")),
+	  dialog = gtk_message_dialog_new (GTK_WINDOW (glade_xml_get_widget (contact->msg_dlg_xml, "message_dialog")),
 					   GTK_DIALOG_DESTROY_WITH_PARENT,
 					   GTK_MESSAGE_WARNING, GTK_BUTTONS_NONE,
 					   _("You can not send messages over 450 characters to offline contacts. Your message was not sent."));
@@ -523,7 +552,7 @@ void send_message (GtkWidget *dlg)
 	g_free(convtext);
 	  
 	
-	v7_sendmsg(mainconnection, kontakt->uin, buf);
+	v7_sendmsg(mainconnection, contact->uin, buf);
 	
 	gtk_text_buffer_set_text(text_buffer, "", -1);
 	
@@ -532,7 +561,7 @@ void send_message (GtkWidget *dlg)
 	  gtk_widget_unrealize(dlg);
 	}
 	else
-	  gtk_widget_grab_focus (glade_xml_get_widget (kontakt->msg_dlg_xml, "input"));
+	  gtk_widget_grab_focus (glade_xml_get_widget (contact->msg_dlg_xml, "input"));
 	
 	/* Show the time of this message in the time widget */
 	now = time (NULL);
@@ -555,9 +584,6 @@ void message_dialog_response_callback (GtkDialog *dlg, gint response, gpointer d
 		case 0: /* Send */
 			send_message (GTK_WIDGET (dlg));
 			break;
-		case 1: /* Read Next */
-			read_next (g_object_get_data (G_OBJECT (dlg), "contact"));
-			break;
 		default:
 			gtk_widget_hide_all (GTK_WIDGET (dlg));
 			gtk_widget_unrealize (GTK_WIDGET (dlg));
@@ -565,44 +591,28 @@ void message_dialog_response_callback (GtkDialog *dlg, gint response, gpointer d
 	}
 }
 
-void show_contact_message( GSList *contact )
+void
+show_contact_message (Contact_Member *contact )
 {
-	STORED_MESSAGE_PTR msg;
+  STORED_MESSAGE_PTR msg;
 
-#ifdef TRACE_FUNCTION
-	g_print("show_contact_message\n");
-#endif
+  g_assert(contact != NULL);
 
-	g_assert(contact != NULL);
+  if (contact->stored_messages) {
+    msg = (STORED_MESSAGE_PTR)contact->stored_messages->data;
+    contact->stored_messages = g_slist_remove (contact->stored_messages, msg);		
+    icq_msgbox (msg, contact->uin);
+    
+    g_free( msg->message );
+    g_free( msg );
 
-	msg = (STORED_MESSAGE_PTR)kontakt->stored_messages->data;
-	if( g_slist_length( kontakt->stored_messages ) )
-	{
-		kontakt->stored_messages = g_slist_remove( kontakt->stored_messages, msg );		
-		icq_msgbox( msg, kontakt->uin );
+  }
 
-		if (msg->type == MESSAGE_URL )
-                  g_free(msg->data);
-		g_free( msg->message );
-		g_free( msg );
+  if (!contact->stored_messages) 
+    gnomeicu_tree_set_contact_icon (contact,
+                                    get_pixbuf_for_status (contact->status));
 
-
-		if( g_slist_length( kontakt->stored_messages ) == 0 )
-		{
-			set_contact_icon (kontakt, get_pixbuf_for_status (kontakt->status));
-			if (kontakt->read_next)
-			     gtk_widget_set_sensitive( kontakt->read_next, FALSE );
-		}
-		else
-		{
-		     if( ((STORED_MESSAGE_PTR)kontakt->stored_messages->data)->type != MESSAGE_TEXT )
-			  gtk_widget_set_sensitive( kontakt->read_next, FALSE);
-		     else
-			  gtk_widget_set_sensitive( kontakt->read_next, TRUE );
-		}
-
-		Shortcut_Update( kontakt );
-	}
+  Shortcut_Update (contact);
 }
 
 gint
@@ -617,29 +627,21 @@ message_input_key_callback (GtkWidget *widget, GdkEventKey *event)
 		send_message (g_object_get_data (G_OBJECT (widget), "message_dialog"));
 		return TRUE;
 	}
-	if ((event->state & GDK_CONTROL_MASK) && (event->keyval == 'n')) {
-		read_next (g_object_get_data (G_OBJECT (widget), "contact"));
-		return TRUE;
-	}
 
 	return FALSE;
 }
 
 void message_usermenu_clicked_callback (GtkWidget *widget, gpointer data)
 {
-	GSList *contact;
-	GtkWidget *personal_menu;
+  Contact_Member *contact;
+  GtkWidget *personal_menu;
 
-#ifdef TRACE_FUNCTION
-	g_print("message_usermenu_click_callback\n");
-#endif
-
-	contact = (GSList*)g_object_get_data (G_OBJECT (widget), "contact");
-	if (contact == NULL)
-		return;
-	personal_menu = user_popup (kontakt);
-	gtk_menu_popup( GTK_MENU( personal_menu ), NULL, NULL, 
-			message_cb_menu_position, widget, 0 ,0);
+  contact = g_object_get_data (G_OBJECT (widget), "contact");
+  if (contact == NULL)
+    return;
+  personal_menu = user_popup (contact);
+  gtk_menu_popup (GTK_MENU (personal_menu), NULL, NULL, 
+                  message_cb_menu_position, widget, 0 ,0);
 }
 
 
@@ -742,6 +744,15 @@ gboolean msg_url_text_button_pressed (GtkWidget *w, GdkEventButton *event,
   if (event->button==1 || event->button==2) {
     char * url = get_url_from_location (w, (int)event->x, (int)event->y);
     if (url != NULL) {
+
+      if (!g_strrstr (url, "http://") && !g_strrstr (url, "https://") && !g_strrstr (url, "ftp://"))
+      {
+        char tmpstr[128] = "http://";
+        strcat(tmpstr, url);
+        g_free (url);
+        url = tmpstr;
+      }
+      
       gnome_url_show(url, NULL);
       g_free (url);
     }
@@ -767,7 +778,14 @@ const char* find_next_url (const char *pch)
 
     if (*pch == 'f') {    
       if (!strncmp (pch, "ftp://", 6))
-	return pch;		       
+	return pch;
+      if (!strncmp (pch, "ftp.", 4))
+          return pch;
+    }
+    
+    if (*pch == 'w') {
+      if (!strncmp (pch, "www.", 4))
+        return pch;
     }
   }
 
@@ -823,62 +841,25 @@ get_url_from_location (GtkWidget * w, int x, int y)
 void message_dialog_focus_callback(GtkWidget *widget, gpointer data)
 {
 
-  GSList *contact = g_object_get_data (G_OBJECT (widget), "contact");
+  Contact_Member *contact = g_object_get_data (G_OBJECT (widget), "contact");
 
-#ifdef TRACE_FUNCTION
-	g_print("message_dialog_focus_callback\n");
-#endif
-
-  if ( kontakt->newmsg_blink_timeout) {
-    gtk_timeout_remove(kontakt->newmsg_blink_timeout);
+  if (contact->newmsg_blink_timeout) {
+    gtk_timeout_remove(contact->newmsg_blink_timeout);
     
-    kontakt->newmsg_blink_timeout = 0;
-    kontakt->newmsg_blink = TRUE;
+    contact->newmsg_blink_timeout = 0;
+    contact->newmsg_blink = TRUE;
 
     msg_blink_timeout(contact);
-
   }
-
 }
 
-void read_next (GSList *contact)
+gboolean on_message_dialog_key_press_event(GtkDialog *dlg, GdkEventKey *event, gpointer data)
 {
-     STORED_MESSAGE_PTR msg;
+  if(event->keyval==GDK_Escape)
+    {
+      gtk_widget_hide_all(GTK_WIDGET(dlg));
+    }
 
-#ifdef TRACE_FUNCTION
-     g_print( "read_next\n" );
-#endif
-
-     if( contact == NULL )
-	  return;
-
-     if (!GTK_WIDGET_SENSITIVE (kontakt->read_next))
-     	return;
-
-     msg = (STORED_MESSAGE_PTR)kontakt->stored_messages->data;
-
-     if( g_slist_length( kontakt->stored_messages ) )
-     {
-	 open_message_dlg_with_message (contact, 0, msg);
-	 g_free( msg->message );
-	 g_free( msg );
-	 kontakt->stored_messages = g_slist_remove( kontakt->stored_messages, msg );
-
-	 if( g_slist_length( kontakt->stored_messages ) == 0 )
-	     {
-		set_contact_icon (kontakt, get_pixbuf_for_status (kontakt->status));
-		gtk_widget_set_sensitive (kontakt->read_next, FALSE);
-	     }
-	 else
-	     {
-		 /* msg points to some strange location already */
-		 if( ((STORED_MESSAGE_PTR)kontakt->stored_messages->data)->type != MESSAGE_TEXT )
-		     gtk_widget_set_sensitive( kontakt->read_next, FALSE );
-		 else
-		     gtk_widget_set_sensitive( kontakt->read_next, TRUE );
-	     }
-	 
-	 Shortcut_Update( kontakt );
-     }
+  return FALSE;
 }
 
