@@ -30,7 +30,7 @@ void notify_list_dialog( void )
 			G_CALLBACK (modify_notify_list),
 			is_notify
 		);
-		gtk_widget_show( dlg );
+		gtk_widget_show_all( dlg );
 		g_object_add_weak_pointer (G_OBJECT (dlg), (gpointer *)&dlg);
 	} else {
 		gtk_window_present (GTK_WINDOW (dlg));
