@@ -16,6 +16,7 @@
 #include "v7send.h"
 
 #include <gtk/gtk.h>
+#include <libgnomeui/gnome-href.h>
 #include <string.h>
 
 void show_personal_info (GtkWidget *widget, gpointer data)
@@ -264,8 +265,18 @@ void update_personal_info ( UIN_T uin )
 	if( kontakt->info->fax != NULL )
 		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "fax_entry")), kontakt->info->fax);
 
-	if( kontakt->info->homepage != NULL )
-		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "homepage_entry")), kontakt->info->homepage);
+	if( (kontakt->info->homepage != NULL) && strlen(kontakt->info->homepage) )
+	{
+	    if( !strstr (kontakt->info->homepage, "://") )
+		string = g_strdup_printf("http://%s", kontakt->info->homepage);
+	    else
+		string = g_strdup(kontakt->info->homepage);
+
+	    gnome_href_set_text (GNOME_HREF(glade_xml_get_widget (kontakt->info_dlg_xml, "homepage_entry")), kontakt->info->homepage);
+	    gnome_href_set_url (GNOME_HREF (glade_xml_get_widget (kontakt->info_dlg_xml, "homepage_entry")), string);
+	    gtk_widget_show (GTK_WIDGET (glade_xml_get_widget (kontakt->info_dlg_xml, "homepage_entry")));
+	    g_free( string );
+	}
 
 	if( kontakt->info->about != NULL )
 	{
@@ -300,8 +311,18 @@ void update_personal_info ( UIN_T uin )
 	if( kontakt->info->work_state != NULL )
 		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "work_state_entry")), kontakt->info->work_state);
 
-	if( kontakt->info->work_homepage != NULL )
-		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "work_homepage_entry")), kontakt->info->work_homepage);
+	if( (kontakt->info->work_homepage != NULL) && strlen(kontakt->info->work_homepage) )
+	{
+	    if( !strstr (kontakt->info->work_homepage, "://") )
+		string = g_strdup_printf("http://%s", kontakt->info->work_homepage);
+	    else
+		string = g_strdup(kontakt->info->work_homepage);
+
+	    gnome_href_set_text (GNOME_HREF(glade_xml_get_widget (kontakt->info_dlg_xml, "work_homepage_entry")), kontakt->info->work_homepage);
+	    gnome_href_set_url (GNOME_HREF (glade_xml_get_widget (kontakt->info_dlg_xml, "work_homepage_entry")), string);
+	    gtk_widget_show (GTK_WIDGET (glade_xml_get_widget (kontakt->info_dlg_xml, "work_homepage_entry")));
+	    g_free( string );
+	}
 
 	if( kontakt->info->work_zip != NULL )
 		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "work_zip_entry")), kontakt->info->work_zip);
