@@ -67,9 +67,12 @@ typedef struct _RTFParserContext {
     /* Header information */
     RTFCharset charset;
     gint codepage;
+    gint default_codepage;
     gint default_font;
     gint default_language;
     
+    gint font_index;
+
     /* Attributes stack management */
     gint group_nesting_level;
     GSList* attributes_stack;
@@ -90,6 +93,10 @@ typedef struct _RTFParserContext {
 
 } RTFParserContext;
 
+typedef struct _RTFFontProperties {
+    gint index;
+    gint charset;
+} RTFFontProperties;
 
 struct codepage_to_locale {
     int codepage;
@@ -124,6 +131,21 @@ rtf_get_int_parameter(RTFParserContext* parser_context, gint* value);
 static void
 rtf_parse_font_table(RTFParserContext* parser_context);
 
+static void
+rtf_parse_font_table_control_word(RTFParserContext* parser_context);
+
+static void
+rtf_parse_charset(RTFParserContext* parser_context, gint charset);
+
+static void
+rtf_set_font_charset(RTFParserContext* parser_context, gint font_index, gint charset);
+
+static void
+rtf_set_font_attributes(RTFParserContext* parser_context, gint font_index);
+
+static void
+rtf_parse_lang(RTFParserContext* parser_context, gint lang);
+
 static void
 rtf_commit_pending_text(RTFParserContext* parser_context);
 
@@ -142,8 +164,6 @@ rtf_push_state(RTFParserContext* parser_context);
 static void
 rtf_parse(RTFParserContext* parser_context, gchar* rtf_text);
 
-
-
 /**********************
  * Exported Functions *
  **********************/
@@ -192,6 +212,8 @@ rtf_parser_context_init(gchar* rtf_text, GtkTextBuffer* text_buffer)
 
     context->charset = CHARSET_UNKNOWN;
     context->codepage = -1;
+    context->default_codepage = -1;
+    context->font_index = 0;
     context->group_nesting_level = 0;
     context->attributes_stack = NULL;
     context->color_table = NULL;
@@ -239,7 +261,7 @@ get_charset_for_codepage(int codepage)
     int i=0;
 
     if (codepage == -1)
-      return g_strdup("ISO-8859-1");
+      return g_strdup(preferences_get_string(PREFS_GNOMEICU_CHARSET_FALLBACK));
 
     /* First try the "CP<cpge>" locale */
     locale = g_strdup_printf("CP%i", codepage);
@@ -357,16 +379,243 @@ rtf_parse_font_table(RTFParserContext* parser_context)
     g_assert(parser_context != NULL);
     /* FIXME : do something with the font table */
 
+    nesting = parser_context->group_nesting_level;
+
+    do {
+        if (*parser_context->current_pos == '{') {
+            parser_context->current_pos++;
+            rtf_push_state(parser_context);
+        } else if (*parser_context->current_pos == '}') {
+            parser_context->current_pos++;
+            rtf_pop_state(parser_context);
+        } else if (*parser_context->current_pos == '\\') {
+            rtf_parse_font_table_control_word(parser_context);
+        } else if (*parser_context->current_pos == '\r') {
+            parser_context->current_pos++;
+        } else {
+            parser_context->current_pos++;
+        }
+
+    } while (parser_context->group_nesting_level >= nesting);
+}
+
+/*
+ * Parses control words in a font table context
+ */
+static void
+rtf_parse_font_table_control_word(RTFParserContext* parser_context)
+{
+    gint param;
+    gchar* word;
+    RTFAttributes* attributes;
+
+    g_assert( parser_context != NULL && *(parser_context->current_pos) == '\\' );
+    attributes = (RTFAttributes*) parser_context->attributes_stack->data;
+
+    word = ++(parser_context->current_pos);
+
+    if (! strncmp("fcharset", word, 8) ) {
+        parser_context->current_pos += 8;
+        if (! rtf_get_int_parameter(parser_context, &param))
+            rtf_warning(parser_context,
+        		"'\\fcharset' control word without parameter.");
+        rtf_parse_charset(parser_context, param); 
+    } else if (! strncmp("f", word, 1) ) {
+      parser_context->current_pos += 1;
+      if (! rtf_get_int_parameter(parser_context, &param))
+	  if (isalpha(parser_context->current_pos[0]))
+	    while (isalpha(parser_context->current_pos[0]))
+	      parser_context->current_pos += 1;
+	  else
+	    rtf_warning(parser_context,
+			"'\\f' control word without parameter.");
+      parser_context->font_index = param;
+    }
+
+    if (*parser_context->current_pos == ' ')
+      parser_context->current_pos++;
+}
+
+/*
+ * Returns properties for font_index or NULL if such font does not exist
+ */
+static RTFFontProperties *
+rtf_get_font_properties(GSList *font_table, gint font_index)
+{
+    gint i;
+    RTFFontProperties *properties;
+
+    for (i = 0; i < g_slist_length(font_table); i++) {
+        properties = g_slist_nth_data(font_table, i);
+        if (properties != NULL && properties->index == font_index)
+            return properties;
+    }
+
+    return NULL;
+}
+
+static void
+rtf_set_font_charset(RTFParserContext* parser_context, gint font_index, gint charset)
+{
+    RTFFontProperties *font_properties = g_new(RTFFontProperties, 1);
+    font_properties->index = font_index;
+    font_properties->charset = charset;
+    parser_context->font_table = g_slist_append(parser_context->font_table, font_properties);
+}
+
+static void
+rtf_set_font_attributes(RTFParserContext* parser_context, gint font_index)
+{
+    RTFFontProperties *font_properties = rtf_get_font_properties(parser_context->font_table,
+                                                                 font_index);
+    if (font_properties)
+        parser_context->codepage = font_properties->charset;
+}
+
+/*
+ * Parses the charset
+ */
+static void
+rtf_parse_charset(RTFParserContext* parser_context, gint charset)
+{
+    gint nesting = 0;
+    char warning[256];
+
+    g_assert(parser_context != NULL);
+    /* FIXME : do something with the font table */
+
+    switch (charset)
+    {
+    case 0:
+    case 1:
+        rtf_set_font_charset(parser_context, parser_context->font_index, parser_context->default_codepage);
+        return;
+    case 130: /* Johab */
+        parser_context->codepage = 1361;
+        break;
+    case 136: /* Big5 */
+    case 1028:
+      parser_context->codepage = 950;
+      break;
+    case 134: /* GB2312 */
+      parser_context->codepage = 936;
+      break;
+    case 161: /* Greek */
+        parser_context->codepage = 1253;
+        break;
+    case 162: /* Turkish */
+        parser_context->codepage = 1254;
+        break;
+    case 163: /* Vietnamese */
+        parser_context->codepage = 1258;
+        break;
+    case 177: /* Hebrew */
+        parser_context->codepage = 1255;
+        break;
+    case 178: /* Arabic */
+    case 179: /* Arabic Traditional */
+    case 180: /* Arabic user */
+        /*
+         * FIXME: which one?
+         * 864 Arabic
+         * 708 Arabic (ASMO 708)
+         * 709 Arabic (ASMO 449+, BCON V4)
+         * 710 Arabic (transparent Arabic)
+         * 711 Arabic (Nafitha Enhanced)
+         * 720 Arabic (transparent ASMO)
+         * 1256 Arabic
+         */
+        snprintf(warning, 256, "charset %d not supported", charset);
+        rtf_warning(parser_context, warning);
+        break;
+    case 181: /* Hebrew user */
+        parser_context->codepage = 862;
+        break;
+    case 186: /* Baltic */
+        parser_context->codepage = 1257;
+        break;
+    case 204: /* Russian */
+        parser_context->codepage = 1251;
+        break;
+    case 222: /* Thai */
+        parser_context->codepage = 874;
+        break;
+    case 238: /* Eastern European */
+        parser_context->codepage = 852;
+        /* ?? 1250 Windows 3.1 (Eastern European) */
+        break;
+    case 254: /* PC 437 */
+        parser_context->codepage = 437;
+        break;
+    case 255: /* OEM */
+        /* FIXME! */
+        /* ...
+         * Other charsets should be recognized here
+         * ...
+         */
+         /*
+          * 819 Windows 3.1 (United States and Western Europe)
+          * 850 IBM multilingual
+          * 860 Portuguese
+          * 863 French Canadian
+          * 865 Norwegian
+          * 866 Soviet Union
+          * 1252 Western European
+          */
+    default:
+        snprintf(warning, 256, "unknown charset: %d", charset);
+        rtf_warning(parser_context, warning);
+        break;
+    }
+
+    rtf_set_font_charset(parser_context, parser_context->font_index, parser_context->codepage);
+
+    /* We don't parse the rest of this statement yet */
+
     while (nesting>=0) {
 	if (*parser_context->current_pos == '{') 
 	    nesting++;
 	if (*parser_context->current_pos == '}')
 	    nesting--;
+
 	parser_context->current_pos++;
     }
     parser_context->current_pos--;
 }
 
+static void
+rtf_parse_lang(RTFParserContext* parser_context, gint lang)
+{
+    gint nesting = 0;
+
+    g_assert(parser_context != NULL);
+
+    switch (lang)
+    {
+    case 9:
+      /* A user said the person sending with lang=9 was using American WinXP */
+    case 1033:
+    case 1036:
+        /* 1036 was reported as french */
+        parser_context->codepage = 1252;
+        break;
+    case 1049:
+        parser_context->codepage = 1251;
+        break;
+    case 1028:
+        parser_context->codepage = 950;
+        break;
+        /* ...
+         * Other charsets should be recognized here
+         * ...
+         */
+    default:
+        rtf_warning(parser_context, "unknown lang, please report the sender's locale and charset");
+        break;
+    }
+}
+
+
 /*
  * Inserts the pending text with the current attributes. This function is 
  * called  whenever a change occurs in the attributes.
@@ -427,9 +676,9 @@ rtf_commit_pending_text(RTFParserContext* parser_context)
       gtk_text_buffer_get_end_iter(parser_context->text_buffer, &text_begin);
       gtk_text_buffer_move_mark(parser_context->text_buffer, 
 				insert_mark, &text_begin);
-    
-      gnomeicu_text_buffer_insert_with_emoticons ( parser_context->text_buffer,
-						   converted_text );
+
+      gnomeicu_text_buffer_insert_with_emoticons (parser_context->text_buffer,
+						  insert_mark, converted_text);
       gtk_text_buffer_get_iter_at_mark(parser_context->text_buffer, 
 				       &text_begin, insert_mark);
       gtk_text_buffer_get_end_iter(parser_context->text_buffer, &text_end);
@@ -596,6 +845,14 @@ rtf_parse_control_word(RTFParserContext* parser_context)
 	    rtf_warning(parser_context, 
 			"'\\ansicpg' control word without parameter.");
 	parser_context->codepage = param;
+        parser_context->default_codepage = param;
+    } else if (! strncmp("lang", word, 4) ) {
+	parser_context->current_pos += 4;
+	if (! rtf_get_int_parameter(parser_context, &param))
+	    rtf_warning(parser_context,
+			"'\\lang' control word without parameter.");
+        rtf_commit_pending_text( parser_context );
+        rtf_parse_lang(parser_context, param);
     } else if (! strncmp("ansi", word, 4) ) {
 	parser_context->current_pos += 4;
 	parser_context->charset = CHARSET_ANSI;
@@ -697,6 +954,12 @@ rtf_parse_control_word(RTFParserContext* parser_context)
 	parser_context->current_pos += 4;
 	/* FIXME : resets paragraph state, but I don't know if this includes
 	   font properties */
+    } else if (! strncmp("keep", word, 4) ) {
+      parser_context->current_pos += 5;
+      /* different type of paragraph separation */
+    } else if (! strncmp("keepn", word, 5) ) {
+      parser_context->current_pos += 5;
+      /* different type of paragraph separation */
     } else if (! strncmp("ulnone", word, 6) ) {
 	parser_context->current_pos += 6;
 	rtf_commit_pending_text( parser_context );
@@ -738,6 +1001,7 @@ rtf_parse_control_word(RTFParserContext* parser_context)
 	g_string_append(attributes->text, buf);
 	rtf_commit_pending_text( parser_context );
 	parser_context->codepage = saved_codepage;
+	dontskip = TRUE;
     } else if (! strncmp("par", word, 3) ) {
 	parser_context->current_pos += 3;
     } else if (! strncmp("line", word, 4) ) {
@@ -786,6 +1050,8 @@ rtf_parse_control_word(RTFParserContext* parser_context)
 	if (! rtf_get_int_parameter(parser_context, &param))
 	    rtf_warning(parser_context, 
 			"'\\f' control word without parameter.");
+        rtf_commit_pending_text( parser_context );
+        rtf_set_font_attributes(parser_context, param);
 	/* FIXME : do something for fonts */
     } else if (! strncmp("b", word, 1) ) {
 	parser_context->current_pos += 1;
@@ -828,7 +1094,7 @@ rtf_parse_control_word(RTFParserContext* parser_context)
 	rtf_get_int_parameter(parser_context, &param);
     }
 
-    if (*parser_context->current_pos == ' ' && !dontskip)
+    if (*parser_context->current_pos == ' ' && !dontskip )
 	parser_context->current_pos++;
 }
 
