@@ -35,7 +35,7 @@ void contact_list_window( CONTACT_PTR contact )
 
 	g_object_set_data(G_OBJECT(g_object_get_data(G_OBJECT(dlg), "button")),
 			  "uin", contact->uin);
-	gtk_widget_show( dlg );
+	gtk_widget_show_all( dlg );
 }
 
 void received_contact_list_window( CONTACT_PTR contact, GSList *contacts )
@@ -69,7 +69,7 @@ void received_contact_list_window( CONTACT_PTR contact, GSList *contacts )
 		contacts = g_slist_remove(contacts, contacts->data);
 	}
 
-	gtk_widget_show( dlg );
+	gtk_widget_show_all( dlg );
 }
 
 void move_contacts_to_list( GtkWidget *widget, gpointer data )
