@@ -28,7 +28,7 @@ static void get_contact_list (GtkWidget *widget, gpointer data);
 static void request_file_from_menu (GtkWidget *widget, gpointer data);
 static void retrieve_away_message( GtkWidget *widget, gpointer data );
 static void add_user_to_list( GtkWidget *widget, gpointer data );
-static void authorize_uin( GtkWidget *widget, gpointer data );
+static void request_auth (GtkWidget *widget, Contact_Member *data);
 static void remove_user_question( GtkWidget *widget, gpointer data );
 static void lists_change_list_ignore (GtkWidget *widget, gpointer data);
 static void lists_change_list_visible (GtkWidget *widget, gpointer data);
@@ -45,17 +45,13 @@ enum {
         ICQ_RESPONSE_SEND = 1
 };
 
-GtkWidget *user_popup (CONTACT_PTR contact)
+GtkWidget *user_popup (Contact_Member *contact)
 {
 	GtkWidget *personal_menu, *lists_menu;
 	GtkWidget *item, *image;
 	GSList *ginfo;
 
-#ifdef TRACE_FUNCTION
-	g_print( "user_popup\n" );
-#endif
-
-	g_assert(contact != NULL);
+	g_assert (contact != NULL);
 
 	personal_menu = gtk_menu_new ();
 
@@ -93,7 +89,7 @@ GtkWidget *user_popup (CONTACT_PTR contact)
 	g_signal_connect (G_OBJECT (item), "activate",
 	                  G_CALLBACK (get_contact_list), contact);
 
-#if 0
+#if 1 /* FXFER */
         /* file transfer */
 	if (contact->used_version >= 8) {
 		item = gtk_image_menu_item_new_with_mnemonic (_("Send _File..."));
@@ -108,6 +104,8 @@ GtkWidget *user_popup (CONTACT_PTR contact)
 	item = gtk_separator_menu_item_new ();
 	gtk_container_add (GTK_CONTAINER (personal_menu), item);
 #endif
+
+        /* user info */
 	item = gtk_image_menu_item_new_with_mnemonic (_("User _Info"));
 	image = gtk_image_new_from_pixbuf (icon_info_pixbuf);
 	gtk_container_add (GTK_CONTAINER (personal_menu), item);
@@ -146,6 +144,7 @@ GtkWidget *user_popup (CONTACT_PTR contact)
 		                  contact);
 	}
 
+        /* history */
 	item = gtk_image_menu_item_new_with_mnemonic (_("_History"));
 	image = gtk_image_new_from_pixbuf (icon_hist_pixbuf);
 	gtk_container_add (GTK_CONTAINER (personal_menu), item);
@@ -236,6 +235,7 @@ GtkWidget *user_popup (CONTACT_PTR contact)
 	gtk_container_add (GTK_CONTAINER (personal_menu), item);
 
 	if (contact->inlist == FALSE) {
+          /* Add to group menu */
 	  lists_menu = gtk_menu_new ();
 
 	  for (ginfo = Groups; ginfo != NULL; ginfo = ginfo->next) {
@@ -291,33 +291,38 @@ GtkWidget *user_popup (CONTACT_PTR contact)
 	}
 
 	/* seperator */
-	item = gtk_separator_menu_item_new ();
-	gtk_container_add (GTK_CONTAINER (personal_menu), item);
+        if (contact->inlist || contact->wait_auth) {
+          item = gtk_separator_menu_item_new ();
+          gtk_container_add (GTK_CONTAINER (personal_menu), item);
+        }
 
+        if (contact->wait_auth) {
+          item = gtk_image_menu_item_new_with_mnemonic (_("Re-request a_uthorization"));
+          image = gtk_image_new_from_pixbuf (icon_auth_pixbuf);
+          gtk_container_add (GTK_CONTAINER (personal_menu), item);
+          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+          gtk_widget_set_sensitive(item, Current_Status != STATUS_OFFLINE);
+          g_signal_connect (G_OBJECT (item), "activate",
+                            G_CALLBACK (request_auth), contact);
+        }
 
-	item = gtk_image_menu_item_new_with_mnemonic (_("A_uthorize User"));
-	image = gtk_image_new_from_pixbuf (icon_auth_pixbuf);
-	gtk_container_add (GTK_CONTAINER (personal_menu), item);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-	gtk_widget_set_sensitive(item, Current_Status != STATUS_OFFLINE);
-	g_signal_connect (G_OBJECT (item), "activate",
-	                  G_CALLBACK (authorize_uin), contact);
+        if (contact->inlist) {
+          item = gtk_image_menu_item_new_with_mnemonic (_("Re_name User..."));
+          image = gtk_image_new_from_pixbuf (icon_rename_pixbuf);
+          gtk_container_add (GTK_CONTAINER (personal_menu), item);
+          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+          gtk_widget_set_sensitive(item, Current_Status != STATUS_OFFLINE);
+          g_signal_connect (G_OBJECT (item), "activate",
+                            G_CALLBACK (change_nick_window), contact);
 
-	item = gtk_image_menu_item_new_with_mnemonic (_("Re_name User..."));
-	image = gtk_image_new_from_pixbuf (icon_rename_pixbuf);
-	gtk_container_add (GTK_CONTAINER (personal_menu), item);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-	gtk_widget_set_sensitive(item, Current_Status != STATUS_OFFLINE);
-	g_signal_connect (G_OBJECT (item), "activate",
-	                  G_CALLBACK (change_nick_window), contact);
-
-	item = gtk_image_menu_item_new_with_mnemonic (_("Re_move User"));
-	image = gtk_image_new_from_pixbuf (icon_cancel_pixbuf);
-	gtk_container_add (GTK_CONTAINER (personal_menu), item);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-	gtk_widget_set_sensitive(item, Current_Status != STATUS_OFFLINE);
-	g_signal_connect (G_OBJECT (item), "activate",
-	                  G_CALLBACK (remove_user_question), contact);
+          item = gtk_image_menu_item_new_with_mnemonic (_("Re_move User"));
+          image = gtk_image_new_from_pixbuf (icon_cancel_pixbuf);
+          gtk_container_add (GTK_CONTAINER (personal_menu), item);
+          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+          gtk_widget_set_sensitive(item, Current_Status != STATUS_OFFLINE);
+          g_signal_connect (G_OBJECT (item), "activate",
+                            G_CALLBACK (remove_user_question), contact);
+        }
 
 	gtk_widget_show_all (personal_menu);
 
@@ -340,9 +345,9 @@ static void icq_sendmessage_window_from_menu( GtkWidget *widget, gpointer data )
 		return;
 
 	if (g_slist_length (kontakt->stored_messages))
-		show_contact_message (contact);
+		show_contact_message (kontakt);
 	else
-		open_message_dlg_with_message (contact, 0, NULL);
+		open_message_dlg_with_message (kontakt, NULL);
 }
 
 static void send_url_window_default( GtkWidget *widget, gpointer data )
@@ -407,7 +412,7 @@ void request_file( GtkWidget *widget, GList *files, gpointer data )
                                               GTK_DIALOG_DESTROY_WITH_PARENT,
                                               NULL);
 
-	gtk_window_set_wmclass (GTK_WINDOW (window), "Send_File", "GnomeICU");
+	gtk_window_set_role (GTK_WINDOW (window), "GnomeICU_SendFile");
 	gtk_window_set_default_size( GTK_WINDOW( window ), 350, 250 );
 	gtk_window_set_policy (GTK_WINDOW (window), FALSE, TRUE, FALSE);
 
@@ -553,32 +558,21 @@ static void add_user_to_list( GtkWidget *widget, gpointer data )
 	if( contact == NULL)
 		return;
 
-	gnomeicu_tree_user_remove(contact);
-
 	v7_try_add_uin (mainconnection, contact->uin, contact->nick, GPOINTER_TO_INT(data), FALSE);
   
 	Save_RC();
 }
 
-static void authorize_uin( GtkWidget *widget, gpointer data )
+static void
+request_auth (GtkWidget *widget, Contact_Member *contact)
 {
-	GSList *contact;
+  g_assert (contact != NULL);
 
-#ifdef TRACE_FUNCTION
-	g_print( "authorize_uin\n" );
-#endif
+  if (!is_connected(GTK_WINDOW(MainData->window),
+                    _("You can not authorize a contact while being disconnected.")))
+    return;
 
-	if (!is_connected(GTK_WINDOW(MainData->window), _("You can not authorize a contact while being disconnected.")))
-		return;
-
-	contact = Contacts;
-	while (contact != NULL && contact->data != data)
-		contact = contact->next;
-
-	if( contact == NULL )
-		return;
-
-        v7_grant_auth(mainconnection, kontakt->uin ); 
+  auth_request_msg_box (contact->nick, contact->uin);
 }
 
 static void remove_user_question( GtkWidget *widget, gpointer data )
@@ -844,9 +838,12 @@ user_change_group (GtkWidget *widget, GroupInfo *d)
 			return;
 
 		uid = v7_user_change_group (mainconnection, contact, d->gid, d->name);
+
 		gnomeicu_tree_user_remove (contact);
 		contact->gid = d->gid;
 		contact->uid = uid;
 		gnomeicu_tree_user_add (contact);
+                if (contact->status != STATUS_OFFLINE)
+                  gnomeicu_tree_user_online (contact);
 	}
 }
