@@ -16,9 +16,15 @@
 #include <libgnomeui/libgnomeui.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netdb.h>
+#include <unistd.h>
+#include <fcntl.h>
 
-void v7_login_connected (GTcpSocket *socket, GInetAddr *ia,
-                   GTcpSocketConnectAsyncStatus status, V7Connection *conn);
+
+gboolean v7_login_connected (GIOChannel *channel, GIOCondition condition,
+			     V7Connection *conn);
 gboolean v7_login_handler (GIOChannel *source, GIOCondition condition,
                            V7Connection *conn);
 gboolean v7_login_error_handler (GIOChannel *iochannel, GIOCondition condition,
@@ -32,6 +38,8 @@ V7Connection *v7_new_login_session(void)
   V7Connection *conn;
   gchar *login_server;
   gint login_port;
+  struct sockaddr_in serv_addr;
+  struct hostent *hp;
 
 #ifdef TRACE_FUNCTION
   g_print( "v7_new_login_session\n" );
@@ -50,56 +58,68 @@ V7Connection *v7_new_login_session(void)
   conn = g_new0(V7Connection, 1);
   conn->last_reqid = 1;
   conn->status = NOT_CONNECTED;
-  conn->gsocket = NULL;
 
   otherconnections = g_slist_append(otherconnections, conn);
 
-  conn->gsocketasyncid = gnet_tcp_socket_connect_async (login_server,
-                                                        login_port,
-                        (GTcpSocketConnectAsyncFunc) v7_login_connected, conn);
 
+  if ((hp = gethostbyname(login_server)) == NULL){
+    g_free (login_server);
+    return NULL;
+  }
   g_free (login_server);
 
+  conn->fd = socket(PF_INET, SOCK_STREAM, 0);
+
+  conn->iochannel = g_io_channel_unix_new(conn->fd);
+
+  conn->in_watch = g_io_add_watch(conn->iochannel, G_IO_IN|G_IO_HUP|G_IO_ERR, 
+				  (GIOFunc)v7_login_connected, conn);
+
+
+  serv_addr.sin_family = AF_INET;
+  serv_addr.sin_port = htons(login_port);
+  memcpy(&serv_addr.sin_addr, hp->h_addr, hp->h_length);
+  
+  fcntl(conn->fd, F_SETFL, fcntl(conn->fd, F_GETFL, 0)|O_NONBLOCK);
+  connect(conn->fd, (struct sockaddr*)&serv_addr, sizeof(serv_addr));
+
+
   return conn;
 
 }
 
-void v7_login_connected (GTcpSocket *socket, GInetAddr *ia,
-              GTcpSocketConnectAsyncStatus status, V7Connection *conn)
+gboolean v7_login_connected (GIOChannel *channel, GIOCondition condition,
+			     V7Connection *conn)
 {
 
 #ifdef TRACE_FUNCTION
   g_print( "v7_login_connected\n" );
 #endif
-
   g_assert(conn != NULL);
 
-  if (status != GTCP_SOCKET_CONNECT_ASYNC_STATUS_OK) {
+  if (condition != G_IO_IN) {
 
     otherconnections = g_slist_remove(otherconnections, conn);
 
+    g_io_channel_shutdown(conn->iochannel, FALSE, NULL);
     g_free(conn);
 
     Current_Status = STATUS_OFFLINE;
-    ready_set(FALSE);
+    gnomeicu_set_status_button (FALSE);
 
-    gnomeicu_spinner_stop();
+    gnomeicu_spinner_stop ();
     isconnecting = FALSE;
-    return;
+    return FALSE;
   }
 
-  conn->gsocket = socket;
-  
   conn->status = CONNECTING;
 
-  conn->iochannel = gnet_tcp_socket_get_iochannel (socket);
-  
   conn->in_watch = g_io_add_watch ( conn->iochannel, G_IO_IN|G_IO_PRI,
                                     (GIOFunc) v7_login_handler, conn);
   conn->err_watch = g_io_add_watch ( conn->iochannel,
                                      G_IO_ERR|G_IO_HUP|G_IO_NVAL,
                                      (GIOFunc) v7_login_error_handler, conn);
-  
+  return FALSE;
 }
 
 gboolean v7_login_error_handler (GIOChannel *iochannel, GIOCondition condition,
@@ -109,6 +129,7 @@ gboolean v7_login_error_handler (GIOChannel *iochannel, GIOCondition condition,
   g_print( "v7_login_error_handler\n" );
 #endif
   icq_set_status_offline( NULL, NULL);
+  
   return FALSE; /* FIXME: not sure which value to return -- menesis */
 }
 
@@ -137,7 +158,7 @@ gboolean v7_login_handler (GIOChannel *iochannel, GIOCondition condition,
 
 
   /* If we dont have the whole packet stop here */
-  if ((packet = read_flap(conn)) == NULL)
+  if ((packet = read_flap(conn)) == NULL) 
     return TRUE;
   
   switch(packet->channel) {
@@ -324,7 +345,7 @@ gboolean v7_login_handler (GIOChannel *iochannel, GIOCondition condition,
 
     g_source_remove(conn->in_watch);
     g_source_remove(conn->err_watch);
-    gnet_tcp_socket_delete(conn->gsocket);
+    g_io_channel_shutdown(conn->iochannel,TRUE, NULL);
     otherconnections = g_slist_remove(otherconnections, conn);
     g_free(conn);
     isconnecting = FALSE;
@@ -351,7 +372,7 @@ gboolean v7_login_handler (GIOChannel *iochannel, GIOCondition condition,
     
     g_source_remove(conn->in_watch);
     g_source_remove(conn->err_watch);
-    gnet_tcp_socket_delete(conn->gsocket);
+    g_io_channel_shutdown(conn->iochannel, TRUE, NULL);
     otherconnections = g_slist_remove(otherconnections, conn);
     g_free(conn);
     isconnecting = FALSE;
