@@ -17,10 +17,14 @@
 
 #include <libgnomeui/libgnomeui.h>
 #include <string.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netdb.h>
+#include <unistd.h>
+#include <fcntl.h>
 
-void v7_newuser_connected (GTcpSocket *socket, GInetAddr *ia,
-                   GTcpSocketConnectAsyncStatus status, V7Connection *conn);
-
+gboolean v7_newuser_connected (GIOChannel *channel, GIOCondition condition,
+			       V7Connection *conn);
 gboolean v7_newuser_handler (GIOChannel *source, GIOCondition condition,
                            V7Connection *conn);
 gboolean v7_newuser_error_handler (GIOChannel *iochannel,
@@ -34,6 +38,8 @@ void v7_new_user(gchar *passwd)
   V7Connection *conn;
   gchar *login_server;
   gint login_port;
+  struct sockaddr_in serv_addr;
+  struct hostent *hp;
 
 #ifdef TRACE_FUNCTION
   g_print( "v7_new_user\n" );
@@ -46,18 +52,39 @@ void v7_new_user(gchar *passwd)
 
   conn = g_new0(V7Connection, 1);
 
-  conn->gsocketasyncid = gnet_tcp_socket_connect_async( login_server, login_port,
-                        (GTcpSocketConnectAsyncFunc) v7_newuser_connected, conn);
+
+
   conn->status = NOT_CONNECTED;
   conn->last_reqid = 1;
   conn->cookie = g_strdup(passwd);
   otherconnections = g_slist_append(otherconnections, conn);
 
+
+  conn->fd = socket(PF_INET, SOCK_STREAM, 0);
+
+  conn->iochannel = g_io_channel_unix_new(conn->fd);
+
+  conn->in_watch = g_io_add_watch(conn->iochannel, G_IO_IN|G_IO_HUP|G_IO_ERR, 
+				  (GIOFunc)v7_newuser_connected, conn);
+
+
+  hp = gethostbyname(login_server);
+
+  serv_addr.sin_family = AF_INET;
+  serv_addr.sin_port = htons(login_port);
+  memcpy(&serv_addr.sin_addr, hp->h_addr, hp->h_length);
+  serv_addr.sin_addr.s_addr = inet_addr(hp->h_addr);
+
+  fcntl(conn->fd, F_SETFL, fcntl(conn->fd, F_GETFL, 0)|O_NONBLOCK);
+  
+  connect(conn->fd, (struct sockaddr*)&serv_addr, sizeof(serv_addr));
+
+
   g_free (login_server);
 }
 
-void v7_newuser_connected (GTcpSocket *socket, GInetAddr *ia,
-              GTcpSocketConnectAsyncStatus status, V7Connection *conn)
+gboolean v7_newuser_connected (GIOChannel *channel, GIOCondition condition,
+			       V7Connection *conn)
 {
 
 #ifdef TRACE_FUNCTION
@@ -66,28 +93,26 @@ void v7_newuser_connected (GTcpSocket *socket, GInetAddr *ia,
 
   g_assert(conn != NULL);
 
-  if (status != GTCP_SOCKET_CONNECT_ASYNC_STATUS_OK) {
-
+  if (condition != G_IO_IN) {
+    g_io_channel_shutdown(conn->iochannel, FALSE, NULL);
+    otherconnections = g_slist_remove(otherconnections, conn);
     g_free(conn);
 
     Current_Status = STATUS_OFFLINE;
-    ready_set(FALSE);
-    gnomeicu_spinner_stop();
+    gnomeicu_set_status_button (FALSE);
+    gnomeicu_spinner_stop ();
     gnome_warning_dialog(_("Can't connect to server\n Adding new user failed"));
-    return;
+    return FALSE;
   }
 
-  conn->gsocket = socket;
-  
   conn->status = CONNECTING;
 
-  conn->iochannel = gnet_tcp_socket_get_iochannel (socket);
-  
   conn->in_watch = g_io_add_watch ( conn->iochannel, G_IO_IN|G_IO_PRI,
                    (GIOFunc) v7_newuser_handler, conn);
   conn->err_watch = g_io_add_watch ( conn->iochannel,
                                      G_IO_ERR|G_IO_HUP|G_IO_NVAL,
                                      (GIOFunc) v7_newuser_error_handler, conn);
+  return FALSE;
 }
   
 gboolean v7_newuser_error_handler (GIOChannel *iochannel,
@@ -156,8 +181,7 @@ gboolean v7_newuser_handler (GIOChannel *iochannel, GIOCondition condition,
       our_info->uin = g_strdup_printf ("%d", Chars_2_DW(snac->data+46));
       passwd = conn->cookie;
 
-      g_source_remove(conn->in_watch);
-      gnet_tcp_socket_delete(conn->gsocket);
+      g_io_channel_shutdown(conn->iochannel, TRUE, NULL);
       otherconnections = g_slist_remove(otherconnections, conn);
       g_free(conn);
 
@@ -172,7 +196,7 @@ gboolean v7_newuser_handler (GIOChannel *iochannel, GIOCondition condition,
       GtkWidget *errordialog;
 
       g_source_remove(conn->in_watch);
-      gnet_tcp_socket_delete(conn->gsocket);
+      g_io_channel_shutdown(conn->iochannel, TRUE, NULL);
       otherconnections = g_slist_remove(otherconnections, conn);
       g_free(conn);
 
