@@ -25,9 +25,14 @@
 #include <libgnomeui/libgnomeui.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netdb.h>
+#include <unistd.h>
+#include <fcntl.h>
 
-void     v7_connected (GTcpSocket *socket, GInetAddr *ia,
-                   GTcpSocketConnectAsyncStatus status, V7Connection *conn);
+gboolean v7_connected (GIOChannel *channel, GIOCondition condition,
+			     V7Connection *conn);
 gboolean v7_handler (GIOChannel *iochannel, GIOCondition condition,
                      V7Connection *conn);
 gboolean v7_error_handler (GIOChannel *iochannel, GIOCondition condition,
@@ -38,7 +43,7 @@ void   v7_snac_channel_handler      (V7Packet *packet, V7Connection *conn);
 void   v7_newconn_channel_handler   (V7Packet *packet, V7Connection *conn);
 
 void   v7_make_requests             (V7Connection *conn);
-void   v7_ack_ack_rate              (V7Connection *conn);
+void   v7_ack_ack_rate              (Snac *snac, V7Connection *conn);
 void   v7_receive_msg               (Snac *snac, V7Connection *conn);
 void   v7_text_message              (UIN_T uin, gchar *here);
 void   v7_special_message           (UIN_T uin, gchar *here);
@@ -83,6 +88,8 @@ void v7_parse_past_and_affiliation_info(Request *requestdat, USER_INFO_PTR info,
 V7Connection *v7_connect(gchar *address, guint port, gchar *cookie, guint cookielen)
 {
   V7Connection *conn;
+  struct sockaddr_in serv_addr;
+  struct hostent *hp;
   
 #ifdef TRACE_FUNCTION
   g_print( "v7_connect\n" );
@@ -93,8 +100,25 @@ V7Connection *v7_connect(gchar *address, guint port, gchar *cookie, guint cookie
   conn->cookie = g_memdup(cookie, cookielen);
   conn->cookielen = cookielen;
 
-  conn->gsocketasyncid = gnet_tcp_socket_connect_async( address, port,
-                             (GTcpSocketConnectAsyncFunc) v7_connected, conn);
+  
+  conn->fd = socket(PF_INET, SOCK_STREAM, 0);
+
+  conn->iochannel = g_io_channel_unix_new(conn->fd);
+
+  conn->in_watch = g_io_add_watch(conn->iochannel, G_IO_IN|G_IO_HUP|G_IO_ERR, 
+				  (GIOFunc)v7_connected, conn);
+
+
+  hp = gethostbyname(address);
+
+  serv_addr.sin_family = AF_INET;
+  serv_addr.sin_port = htons(port);
+  memcpy(&serv_addr.sin_addr, hp->h_addr, hp->h_length);
+
+  fcntl(conn->fd, F_SETFL, fcntl(conn->fd, F_GETFL, 0)|O_NONBLOCK);
+
+  connect(conn->fd, (struct sockaddr*)&serv_addr, sizeof(serv_addr));
+
   conn->last_reqid = 1;
   conn->status = NOT_CONNECTED;
 
@@ -103,39 +127,34 @@ V7Connection *v7_connect(gchar *address, guint port, gchar *cookie, guint cookie
 
 
 
-void v7_connected (GTcpSocket *socket, GInetAddr *ia,
-                   GTcpSocketConnectAsyncStatus status, V7Connection *conn)
-
+gboolean v7_connected (GIOChannel *channel, GIOCondition condition,
+		       V7Connection *conn)
 {
 
 #ifdef TRACE_FUNCTION
   g_print( "v7_connected\n" );
 #endif
-  
-  if (status != GTCP_SOCKET_CONNECT_ASYNC_STATUS_OK) {
-
+  if (condition != G_IO_IN) {
+    g_io_channel_shutdown(conn->iochannel, FALSE, NULL);
     g_free(conn);
     Current_Status = STATUS_OFFLINE;
-    ready_set(FALSE);
+    gnomeicu_set_status_button (FALSE);
     enable_online_events = FALSE;
-    gnomeicu_spinner_stop();
+    gnomeicu_spinner_stop ();
 
-    return;
+
+    return FALSE;
   }
   
   
-  conn->gsocket = socket;
-  
   conn->status = CONNECTING;
-  
-  conn->iochannel = gnet_tcp_socket_get_iochannel (socket);
-  
   conn->in_watch = g_io_add_watch ( conn->iochannel, G_IO_IN|G_IO_PRI,
                                     (GIOFunc) v7_handler, conn);
   conn->err_watch = g_io_add_watch ( conn->iochannel,
                                      G_IO_ERR|G_IO_HUP|G_IO_NVAL,
                                      (GIOFunc) v7_error_handler, conn);
-  
+
+  return FALSE;
 }
 
 gboolean v7_error_handler (GIOChannel *iochannel, GIOCondition condition,
@@ -146,7 +165,7 @@ gboolean v7_error_handler (GIOChannel *iochannel, GIOCondition condition,
 #endif
 
   icq_set_status_offline( NULL, NULL);
-  return FALSE; /* not sure what to return -- menesis */
+  return FALSE;
 }
 
 gboolean v7_handler (GIOChannel *iochannel, GIOCondition condition,
@@ -214,7 +233,7 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
       v7_iam_icq(conn);
       break;
     case F01_SERVER_ACK_RATE:
-      v7_ack_ack_rate(conn);
+      v7_ack_ack_rate(snac, conn);
       v7_make_requests(conn);
       break;
     case F01_SERVER_SENDING_TOO_FAST:
@@ -323,6 +342,12 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
     break;
   case FAMILY_15_SAVED_DATA:
     switch (snac->type) {
+    case F15_BOTH_ERROR:
+      switch(CharsBE_2_Word(snac->data)) {
+      case 0x02: /* Rate too fast */
+	g_print("Server says we are sending too fast..\n");
+	break;
+      }
     case F15_SERVER_SAVED_DATA:
       v7_saved_data(conn, snac);
     default:
@@ -342,7 +367,7 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
 
   g_free (snac->data);
   g_free(snac);
-  
+
 }
 
 
@@ -380,16 +405,125 @@ void v7_newconn_channel_handler (V7Packet *packet, V7Connection *conn)
 
 
 
-void v7_ack_ack_rate    (V7Connection *conn)
+void v7_ack_ack_rate    (Snac *snac, V7Connection *conn)
 {
-  BYTE v7_ack_ack_rate_array[10] = "\x00\x01\x00\x02\x00\x03\x00\x04\x00\x05";
+  WORD classcount, i, j, classid, pairscount, family, subfamily;
+  DWORD *familyandsub;
+  BYTE state;
+  GList *currentclass;
+  V7RateClass* class;
+  TLVstack *tlvs = new_tlvstack(NULL, 0);
+  gchar *here;
+
 
 #ifdef TRACE_FUNCTION
   g_print( "v7_ack_ack_rate\n" );
 #endif
 
-  snac_send(conn, v7_ack_ack_rate_array, 10, FAMILY_01_GENERIC,
-            F01_CLIENT_ACK_ACK_RATE, NULL, ANY_ID);
+  classcount = CharsBE_2_Word(snac->data);
+  
+
+  here = snac->data + 2;
+  
+  for(i=0; i < classcount; i++) {
+    if ( ((int)here)-((int)snac->data)+33 > snac->datalen) {
+      g_warning("Bizzare ack rate packet, parsing interupted half way through");
+      break;
+    }
+
+    classid = CharsBE_2_Word(here);
+    here += 2;
+
+    currentclass = conn->rateclasses;
+    while(currentclass != NULL &&
+	  ((V7RateClass*)currentclass->data)->id != classid)
+      currentclass = currentclass->next;
+    if (currentclass) 
+      class = currentclass->data;
+    else {
+      class = g_new0(V7RateClass,1);
+      class->id = classid;
+      conn->rateclasses = g_list_append(conn->rateclasses, class);
+    }
+
+    class->windowsize = CharsBE_2_DW(here);
+    here += 4;
+    
+    class->clearlevel = CharsBE_2_DW(here);
+    here += 4;
+    
+    class->alertlevel = CharsBE_2_DW(here);
+    here += 4;
+    
+    class->limitlevel = CharsBE_2_DW(here);
+    here += 4;
+    
+    class->disconnectlevel = CharsBE_2_DW(here);
+    here += 4;
+    
+    class->currentlevel = CharsBE_2_DW(here);
+    here += 4;
+  
+    class->maxlevel = CharsBE_2_DW(here);
+    here += 4;
+    
+    class->lasttime = CharsBE_2_DW(here);
+    here += 4;
+    
+    class->state = *here;
+    here += 1;
+
+    /*    g_print("Class: %d State: %d\n", class->id, class->state);
+	  g_print("WinSize: %d Clear: %d Alert: %d Limit: %d Discon: %d Avg: %d MaxAvg:%d LastTime: %d\n", class->windowsize, class->clearlevel, class->alertlevel, class->limitlevel, disconnectlevel, class->currentlevel, class->maxlevel, class->lasttime); */
+
+    add_nontlv_w_be(tlvs, classid);
+  }
+  
+  if (classcount && !conn->familyclass) {
+    conn->familyclass = g_hash_table_new_full(g_int_hash, g_int_equal, g_free,
+					      NULL);
+  }
+
+  for(i=0; i < classcount; i++) {
+    if ( ((int)here)-((int)snac->data) > snac->datalen) {
+      g_warning("Bizzare ack rate packet, parsing interupted half way through");
+      break;
+    }
+    classid = CharsBE_2_Word(here);
+    here += 2;
+
+    currentclass = conn->rateclasses;
+    while(currentclass != NULL &&
+	  ((V7RateClass*)currentclass->data)->id != classid)
+      currentclass = currentclass->next;
+    
+    if (!currentclass) {
+      g_warning("Trying to add family/subfamily pairs to non-existing rate class: %d\n", classid);
+      continue;
+    }
+
+    pairscount = CharsBE_2_Word(here);
+    here += 2;
+    
+    for (j=0; j < pairscount; j++) {
+      familyandsub = g_new0(DWORD,1);
+      *familyandsub = CharsBE_2_DW(here);
+      family = CharsBE_2_Word(here);
+      here += 2;
+      subfamily = CharsBE_2_Word(here);
+      here += 2;
+      g_hash_table_replace(conn->familyclass, familyandsub, 
+			   currentclass->data);
+      //      g_print("C: %d F: %X/%X\n", classid, family, subfamily);
+    }
+    
+  }
+
+  if (classcount)
+    snac_send(conn, tlvs->beg, tlvs->len, FAMILY_01_GENERIC,
+	      F01_CLIENT_ACK_ACK_RATE, NULL, ANY_ID);
+
+  free_tlvstack(tlvs);
 }
 
 void v7_receive_msg (Snac *snac, V7Connection *conn)
@@ -578,14 +712,14 @@ void v7_special_message(UIN_T uin, gchar *here)
     *tmp = '\0';
     tmp++;
 
-    url = tmp;
+    msg = tmp;
     tmp = strchr( tmp, '\xFE' );
     if ( tmp == NULL )
       return; /* Bad Packet */
     *tmp = '\0';
     tmp ++;
 
-    authorization_request(uin, nick, first, last, email, url, msg_time);
+    authorization_request(uin, msg,msg_time);
     break;
   default:
     g_warning(_("Special message type not handled, type: %x, text: %s\n"), msgtype, msg);
@@ -600,7 +734,7 @@ void v7_advanced_message(UIN_T uin, V7Connection *conn, gchar *data, int len,
 {
 
   TLV *maintlv, *insidetlv, *tlv;
-  WORD type1, msglen, fileport;
+  WORD type1, msglen, fileport, downcounter;
   BYTE msgtype;
   gchar *msg=NULL, *here, *url, *nick, *first, *last, *email, *tmp=NULL, *file,
     *msgid, *conv_msg;
@@ -667,8 +801,11 @@ void v7_advanced_message(UIN_T uin, V7Connection *conn, gchar *data, int len,
     delete_tlv(insidetlv);
     insidetlv = new_tlv(here);
   }
-  
-  
+
+  /* reading tlv 0x2711 */
+
+  downcounter = Chars_2_Word (insidetlv->value + 29 + 2);
+
   msgtype = insidetlv->value[45];
   
   if (msgtype != 0x1A) {
@@ -689,6 +826,13 @@ void v7_advanced_message(UIN_T uin, V7Connection *conn, gchar *data, int len,
   case 0x13: /* Contacts */
     
     if (isfile) {
+      if (isfileokack) {
+	tlv = new_tlv(here+insidetlv->len+4);
+	if (tlv->type == 4 && tlv->len == 4)
+	  fileip = IP_2_DW(tlv->value);
+	delete_tlv(tlv);
+      }
+
       here = insidetlv->value+54;
       msglen =  Chars_2_Word(here);
       here += 2 + msglen + 4;
@@ -705,20 +849,15 @@ void v7_advanced_message(UIN_T uin, V7Connection *conn, gchar *data, int len,
       filesize = Chars_2_DW(here);
       here += 4;
       fileport = Chars_2_Word(here);
-      here += 4;
-
-      tlv = new_tlv(here);
-      if (tlv->type == 4 && tlv->len == 4)
-	fileip = IP_2_DW(tlv->value);
-      delete_tlv(tlv);
+      here += 2;
 
       msgid = g_memdup(begofmes, 8);
 
-      /*    g_print("msg: %s\n file:%s\n size:%d ip:%lX port:%d \n", msg,
-	    file, filesize, fileip, fileport); */
+      /* g_print("msg: %s\n file:%s\n size:%d ip:%lX port:%d count:0x%x\n", msg,
+         file, filesize, fileip, fileport, downcounter); */
 
       if (! isfileokack)
-	Do_File( msg, uin, msgid, file, filesize, conn);
+	Do_File( msg, uin, msgid, file, filesize, conn, downcounter);
       else 
 	start_transfer(uin, msgid, fileip, fileport);
       
@@ -784,7 +923,7 @@ void v7_advanced_message(UIN_T uin, V7Connection *conn, gchar *data, int len,
     *tmp = '\0';
     tmp++;
 
-    url = tmp;
+    msg = tmp;
     tmp = strchr( tmp, '\xFE' );
     if ( tmp == NULL ) {
       g_free(msg);
@@ -793,7 +932,7 @@ void v7_advanced_message(UIN_T uin, V7Connection *conn, gchar *data, int len,
     *tmp = '\0';
     tmp ++;
 
-    authorization_request(uin, nick, first, last, email, url, msg_time);
+    authorization_request(uin, msg, msg_time);
     break;
   case 0xE8:  /* Request for away message */
   case 0xE9:
@@ -910,6 +1049,13 @@ void v7_incoming_user (Snac *snac)
       break;
     case 0x06:
       tempstatus = CharsBE_2_Word(tlv->value+2);
+
+      /* Check for birthday */
+      if((CharsBE_2_Word(tlv->value) & 0x08) == 0x08)
+	kontakt->has_birthday = TRUE;      
+      else 
+	kontakt->has_birthday = FALSE;
+
       if (kontakt->status != tempstatus)
       	status_has_changed = TRUE;
       /* getting rid of non-standard statuses here */
@@ -976,7 +1122,7 @@ void v7_acked_adv_msg (Snac *snac, V7Connection *conn)
 {
 
   BYTE type;
-  gchar *awaymessage;
+  gchar *awaymessage, *awaymessage_utf8;
   UIN_T sender_uin;
   WORD sender_status;
 
@@ -1011,7 +1157,11 @@ void v7_acked_adv_msg (Snac *snac, V7Connection *conn)
   awaymessage = g_strndup(snac->data+11+snac->data[10]+2+26+1+20+1+3+2,
                           Chars_2_Word(snac->data+11+snac->data[10]+2+26+1+20+1+3));
   
-  recv_awaymsg( sender_uin, sender_status, awaymessage);
+  awaymessage_utf8 = convert_to_utf8(awaymessage);
+  g_free(awaymessage);
+
+  recv_awaymsg( sender_uin, sender_status, awaymessage_utf8);
+  g_free(awaymessage_utf8);
 }
 
 void v7_saved_data(V7Connection *conn, Snac *snac)
@@ -1252,14 +1402,14 @@ void v7_offline_message(DWORD reqid, gchar *data)
     *tmp = '\0';
     tmp++;
 
-    url = tmp;
+    msg = tmp;
     tmp = strchr( tmp, '\xFE' );
     if ( tmp == NULL )
       return; /* Bad Packet */
     *tmp = '\0';
     tmp ++;
 
-    authorization_request(uin, nick, first, last, email, url, msg_time);
+    authorization_request(uin, msg, msg_time);
     break;
   case 0x13:
     v7_rec_contactlist(uin, msg, msglen, msg_time);
@@ -1386,7 +1536,7 @@ void v7_parse_main_home_info (Request *requestdat, USER_INFO_PTR info,
   g_free(info->nick);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->nick = g_strdup( here );
+  info->nick = convert_to_utf8( here );
   here += len_str;
   if (requestdat->type == NEW_USER_VERIFICATION) {
     GSList *contact = Contacts;
@@ -1402,61 +1552,61 @@ void v7_parse_main_home_info (Request *requestdat, USER_INFO_PTR info,
   g_free(info->first);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->first = g_strdup( here );
+  info->first = convert_to_utf8( here );
   here += len_str;
 
   g_free(info->last);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->last = g_strdup( here );
+  info->last = convert_to_utf8( here );
   here += len_str;
 
   g_free(info->email);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->email = g_strdup( here );
+  info->email = convert_to_utf8( here );
   here += len_str;
 
   g_free(info->city);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->city = g_strdup( here );
+  info->city = convert_to_utf8( here );
   here += len_str;
 
   g_free(info->state);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->state = g_strdup( here );
+  info->state = convert_to_utf8( here );
   here += len_str;
 
   g_free(info->phone);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->phone = g_strdup( here );
+  info->phone = convert_to_utf8( here );
   here += len_str;
 
   g_free(info->fax);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->fax = g_strdup( here );
+  info->fax = convert_to_utf8( here );
   here += len_str;
 
   g_free(info->street);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->street = g_strdup( here );
+  info->street = convert_to_utf8( here );
   here += len_str;
 
   g_free(info->cellular);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->cellular = g_strdup( here );
+  info->cellular = convert_to_utf8( here );
   here += len_str;
 
   g_free(info->zip);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->zip = g_strdup( here );
+  info->zip = convert_to_utf8( here );
   here += len_str;
 
   info->country = Chars_2_Word( here );
@@ -1504,12 +1654,19 @@ void v7_parse_more_info (Request *requestdat, USER_INFO_PTR info,
   g_free(info->homepage);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->homepage = g_strdup( here );
+  info->homepage = convert_to_utf8( here );
   here += len_str;
 
   info->birth_year = Chars_2_Word(here);
   info->birth_month = *(here+2);
   info->birth_day = *(here+3);
+
+  our_info->birth_year = info->birth_year;
+  our_info->birth_month = info->birth_month;
+  our_info->birth_day = info->birth_day;
+
+  v7_setstatus2(mainconnection, Current_Status);
+
   here += 4;
   info->language1 = here[0];
   info->language2 = here[1];
@@ -1577,13 +1734,13 @@ void v7_parse_work_info (Request *requestdat, USER_INFO_PTR info,
   g_free(info->work_phone);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->work_phone = g_strdup( here );
+  info->work_phone = convert_to_utf8( here );
   here += len_str;
 
   g_free(info->work_fax);
   len_str = Chars_2_Word( here );
   here += 2;
-  info->work_fax = g_strdup( here );
+  info->work_fax = convert_to_utf8( here );
   here += len_str;
 
   g_free(info->work_address);
@@ -1639,7 +1796,9 @@ void v7_parse_emails_info(Request *requestdat, USER_INFO_PTR info,
                           BYTE result, gchar *here)
 {
   int len_str, count, i;
+  int len_temp;
   gchar *email;
+  gchar *temp;
   GList *element;
 
 #ifdef TRACE_FUNCTION
@@ -1662,13 +1821,18 @@ void v7_parse_emails_info(Request *requestdat, USER_INFO_PTR info,
     here++; /* Skipping hide, if it was hidden, we wouldnt see it */
     len_str = Chars_2_Word( here );
     here += 2;
-    email = g_malloc0(len_str+1);
+    temp = convert_to_utf8(here);
+    if (temp != NULL)
+      len_temp = strlen(temp)+1;
+    else
+      len_temp = len_str;
+    email = g_malloc0(len_temp+1);
     email[0] = here[-3]; /* save the hide value for use if it's our info */
-    strncpy( email+1, here, len_str );
+    strncpy( email+1, (temp != NULL) ? temp : here , len_temp );
     here += len_str;
+    g_free(temp);
     
-    info->emails = g_list_append(info->emails,
-                                              email);
+    info->emails = g_list_append(info->emails, email);
   }
 
   if (requestdat->type == OUR_INFO) {
@@ -1681,7 +1845,9 @@ void v7_parse_homepage_category_info(Request *requestdat, USER_INFO_PTR info,
                           BYTE result, gchar *here)
 {
   int len_str, count, i;
+  int len_temp;
   gchar *category;
+  gchar *temp;
   GList *element;
 
 #ifdef TRACE_FUNCTION
@@ -1704,10 +1870,16 @@ void v7_parse_homepage_category_info(Request *requestdat, USER_INFO_PTR info,
     here += 2; /* skipping category for the moment */
     len_str = Chars_2_Word( here );
     here += 2;
-    category = g_malloc0(len_str+2);
+    temp = convert_to_utf8(here);
+    if (temp != NULL)
+      len_temp = strlen(temp)+1;
+    else
+      len_temp = len_str;
+    category = g_malloc0(len_temp+2);
     *(WORD *) category = Chars_2_Word(here-4);
-    strncpy( category+2, here, len_str );
+    strncpy( category+2, (temp != NULL) ? temp : here, len_temp );
     here += len_str;
+    g_free(temp);
     
     info->homepage_cat = g_list_append(info->homepage_cat, category);
   }
@@ -1722,7 +1894,9 @@ void v7_parse_interests_info(Request *requestdat, USER_INFO_PTR info,
                           BYTE result, gchar *here)
 {
   int len_str, count, i;
+  int len_temp;
   gchar *interest;
+  gchar *temp;
   GList *element;
 
 #ifdef TRACE_FUNCTION
@@ -1745,10 +1919,16 @@ void v7_parse_interests_info(Request *requestdat, USER_INFO_PTR info,
     here += 2; /* skipping category for the moment */
     len_str = Chars_2_Word( here );
     here += 2;
-    interest = g_malloc0(len_str+2);
+    temp = convert_to_utf8(here);
+    if (temp != NULL)
+      len_temp = strlen(temp)+1;
+    else
+      len_temp = len_str;
+    interest = g_malloc0(len_temp+2);
     *(WORD *) interest = Chars_2_Word(here-4);
-    strncpy( interest+2, here, len_str );
+    strncpy( interest+2, (temp != NULL) ? temp : here, len_temp );
     here += len_str;
+    g_free(temp);
     
     info->interests = g_list_append(info->interests, interest);
   }
@@ -1763,7 +1943,9 @@ void v7_parse_past_and_affiliation_info(Request *requestdat, USER_INFO_PTR info,
                           BYTE result, gchar *here)
 {
   int len_str, count, i;
+  int len_temp;
   gchar *buffer;
+  gchar *temp;
   GList *element;
 
 #ifdef TRACE_FUNCTION
@@ -1786,10 +1968,16 @@ void v7_parse_past_and_affiliation_info(Request *requestdat, USER_INFO_PTR info,
     here += 2; /* skipping category for the moment */
     len_str = Chars_2_Word( here );
     here += 2;
-    buffer = g_malloc0(len_str+2);
+    temp = convert_to_utf8(here);
+    if (temp != NULL)
+      len_temp = strlen(temp)+1;
+    else
+      len_temp = len_str;
+    buffer = g_malloc0(len_temp+2);
     *(WORD *) buffer = Chars_2_Word(here-4);
-    strncpy( buffer+2, here, len_str );
+    strncpy( buffer+2, (temp != NULL) ? temp : here, len_temp );
     here += len_str;
+    g_free(temp);
     
     info->past_background = g_list_append(info->past_background, buffer);
   }
@@ -1810,10 +1998,16 @@ void v7_parse_past_and_affiliation_info(Request *requestdat, USER_INFO_PTR info,
     here += 2; /* skipping category for the moment */
     len_str = Chars_2_Word( here );
     here += 2;
-    buffer = g_malloc0(len_str+2);
+    temp = convert_to_utf8(here);
+    if (temp != NULL)
+      len_temp = strlen(temp)+1;
+    else
+      len_temp = len_str;
+    buffer = g_malloc0(len_temp+2);
     *(WORD *) buffer = Chars_2_Word(here-4);
-    strncpy( buffer+2, here, len_str );
+    strncpy( buffer+2, (temp != NULL) ? temp : here, len_temp );
     here += len_str;
+    g_free(temp);
     
     info->affiliations = g_list_append(info->affiliations, buffer);
   }
@@ -1876,7 +2070,9 @@ void v7_rec_contactlist(UIN_T uin, gchar *msg, int msglen, time_t msg_time)
 void  v7_rate_changed (Snac *snac, V7Connection *conn)
 {
 
-  WORD code, class;
+  WORD code, classid;
+  V7RateClass *class;
+  GList *currentclass;
   DWORD window_size, clear, alert, limit, disconnect, currentavg, maxavg;
   gchar *here;
   
@@ -1896,35 +2092,54 @@ void  v7_rate_changed (Snac *snac, V7Connection *conn)
     4 = Limit cleared
   */
 
-  class = CharsBE_2_Word(here);
+  classid = CharsBE_2_Word(here);
+
   here += 2;
-  
-  window_size = CharsBE_2_DW(here);
-  here += 4;
-  
-  clear = CharsBE_2_DW(here);
-  here += 4;
-  
-  alert = CharsBE_2_DW(here);
-  here += 4;
-  
-  limit = CharsBE_2_DW(here);
-  here += 4;
-  
-  disconnect = CharsBE_2_DW(here);
-  here += 4;
-  
-  currentavg = CharsBE_2_DW(here);
-  here += 4;
-  
-  maxavg = CharsBE_2_DW(here);
-  here += 4;
+  currentclass = conn->rateclasses;
+  while(currentclass != NULL &&
+	((V7RateClass*)currentclass->data)->id != classid)
+    currentclass = currentclass->next;
+  if (currentclass) 
+    class = currentclass->data;
+  else {
+    g_warning("The server sent class changing information for an unknown class\n");
+    return;
+  }
 
-  g_print("Code: %d Class: %d\n", code, class);
-  g_print("WinSize: %d Clear: %d Alert: %d Limit: %d Discon: %d Avg: %d MaxAvg:%d\n", window_size, clear, alert, limit, disconnect, currentavg, maxavg);
+  
+  class->windowsize = CharsBE_2_DW(here);
+  here += 4;
+  
+  class->clearlevel = CharsBE_2_DW(here);
+  here += 4;
+  
+  class->alertlevel = CharsBE_2_DW(here);
+  here += 4;
+  
+  class->limitlevel = CharsBE_2_DW(here);
+  here += 4;
+  
+  class->disconnectlevel = CharsBE_2_DW(here);
+  here += 4;
+  
+  class->currentlevel = CharsBE_2_DW(here);
+  here += 4;
+  
+  class->maxlevel = CharsBE_2_DW(here);
+  here += 4;
+  
+  class->lasttime = CharsBE_2_DW(here);
+  here += 4;
+  
+  class->state = *here;
+  here += 1;
 
-  if (code == 2)
-    gnome_warning_dialog(_("You are sending too fast, please slow down your sending speed."));
+    //    g_print("Class:
+  //  g_print("Code: %d Class: %d\n", code, class);
+  // g_print("WinSize: %d Clear: %d Alert: %d Limit: %d Discon: %d Avg: %d MaxAvg:%d\n", window_size, clear, alert, limit, disconnect, currentavg, maxavg);
+
+  //  if (code == 2)
+  //    gnome_warning_dialog(_("You are sending too fast, please slow down your sending speed."));
   
   if (code == 3)
     gnome_warning_dialog(_("You are sending messages too fast, your last message has been dropped, please wait 10 seconds and resend. If you continue sending messages at that speed, you will be disconnect by the server. Please give it a break."));
@@ -1944,6 +2159,8 @@ void v7_server_status(Snac *snac, V7Connection *conn)
 
   here = snac->data;
 
+  if (here[0] == 0) here += 8;
+
   here += here[0] + 1;
   
   here += 2; /* Warning level ??? */
