@@ -29,8 +29,9 @@ static void v7_sendmsg4(V7Connection *conn, UIN_T uin, gchar *msgcontent, int co
 static DWORD v7_send_15_02_D007( V7Connection *conn, WORD subtype, gchar *buffer, int buffer_len);
 static void v7_send_simple_msg2(V7Connection *conn, UIN_T uin, const gchar *msg, BYTE type, gboolean autoreq);
 static void v7_send_file_msg2(V7Connection *conn, UIN_T uin, const gchar *msg,
-		       WORD port, DWORD ip, gboolean filereq,
-		       const gchar *filename, DWORD filesize, gchar *msgid);
+			      WORD port, DWORD ip, gboolean filereq,
+			      const gchar *filename, DWORD filesize, 
+			      gchar *msgid, WORD downcounter);
 
 const gchar zerobuf[20] = "\0\0\0\0\0\0\0\0\0\0" "\0\0\0\0\0\0\0\0\0\0";
 
@@ -40,6 +41,9 @@ void v7_sendmsg(V7Connection *conn, UIN_T uin, const guchar *text)
 
   contact = Find_User(uin);
 
+  if (contact == NULL)
+    return;
+
   history_add_outgoing( uin, text );
 
   if (kontakt->has_rtf)
@@ -115,7 +119,7 @@ void v7_sendmsg_rtf(V7Connection *conn, UIN_T uin, const guchar *text)
 	g_string_append_c(string, *newtext);
     else {
       unicodetemp = g_utf8_get_char(newtext);
-      g_string_append_printf(string, "\\u%d? ", unicodetemp);
+      g_string_append_printf(string, "\\u%d?", unicodetemp);
     }
     
   } 
@@ -139,7 +143,7 @@ void v7_setstatus(V7Connection *conn, DWORD newstatus)
 	if (newstatus == STATUS_ONLINE || newstatus == STATUS_FREE_CHAT)
 	  v7_aimsetstatus (mainconnection, NULL);
 	Current_Status = newstatus;
-	ready_set(FALSE);
+	gnomeicu_set_status_button (FALSE);
   	return;
   }
 
@@ -159,10 +163,24 @@ void v7_setstatus2(V7Connection *conn, DWORD newstatus)
 {
   TLVstack *tlvs;
   gchar directconnectioninfo[37];
+  time_t today;
+  struct tm *today2;
 
   newstatus |= (our_info->webpresence ? 0x10000 : 0) |
     0x20000 | 0x10000000;
-  /* 0x0008 == BIRTHDAY flag */
+
+
+  /* Check birthday to set the flag */
+
+  today = time(NULL);
+  today2 = localtime(&today);
+
+  if(our_info->birth_day == 0)
+    v7_request_our_info(mainconnection);
+
+  if((our_info->birth_day == today2->tm_mday) && 
+     (our_info->birth_month == today2->tm_mon+1))
+      newstatus |=  0x80000;
 
   tlvs = new_tlvstack(NULL,0);
   
@@ -192,18 +210,20 @@ void v7_setstatus2(V7Connection *conn, DWORD newstatus)
   
   free_tlvstack(tlvs);
 
-  ready_set(FALSE);
+  gnomeicu_set_status_button (FALSE);
 }
 
 void v7_quit()
 {
 
+  GList *list;
+
 #ifdef TRACE_FUNCTION
   g_print( "v7_quit\n" );
 #endif 
 
   if (mainconnection) {
-
+    
     if (mainconnection->in_watch)
       if (!g_source_remove(mainconnection->in_watch)) {
         gnome_error_dialog(_("Cannot remove main input watcher!"));
@@ -216,8 +236,27 @@ void v7_quit()
         return;
       }
 
-    if (mainconnection->gsocket)
-      gnet_tcp_socket_delete(mainconnection->gsocket);
+
+    if (mainconnection->iochannel)
+      g_io_channel_shutdown(mainconnection->iochannel, TRUE, NULL);
+
+    list = mainconnection->queued_packets;
+    while (list) {
+      gtk_timeout_remove(((RateLimitedResendData*)list->data)->timerid);
+      g_free(((RateLimitedResendData*)list->data)->buffer);
+      g_free(list->data);
+      list = list->next;
+    }
+    g_list_free(mainconnection->queued_packets);
+    
+    list = mainconnection->rateclasses;
+    while (list) {
+      g_free(list->data);
+      list = list->next;
+    }
+    g_list_free(mainconnection->rateclasses);
+    
+    g_hash_table_destroy(mainconnection->familyclass);
     
     g_free(mainconnection);
     
@@ -236,8 +275,9 @@ void v7_quit()
         gnome_error_dialog(_("Cannot remove other error watcher!"));
         return;
      }
-    if (((V7Connection*)otherconnections->data)->gsocket)
-      gnet_tcp_socket_delete(((V7Connection*)otherconnections->data)->gsocket);
+    if (((V7Connection*)otherconnections->data)->iochannel)
+      g_io_channel_shutdown(((V7Connection*)otherconnections->data)->iochannel,
+			    TRUE, NULL);
     
     g_free(otherconnections->data);
     otherconnections = g_slist_remove(otherconnections, otherconnections->data);
@@ -403,8 +443,7 @@ void v7_send_simple_msg2(V7Connection *conn, UIN_T uin, const gchar *msg, BYTE t
 #endif 
  
   middletlvs = new_tlvstack("\0\0" "\041\032\031\042" "\063\035\063\024"
-                           "\x09\x46\x13\x49\x4C\x7F\x11\xD1"
-                           "\x82\x22\x44\x45\x53\x54\x00\x00", 16+8+2);
+                           CAP_AIM_SERVERRELAY, 16+8+2);
 
   
   add_tlv(middletlvs, 0x0A, "\x00\x01", 2);
@@ -473,8 +512,7 @@ void v7_send_type1_msg2(V7Connection *conn, UIN_T uin, const gchar *msg)
 #endif 
  
   middletlvs = new_tlvstack("\0\0" "\041\032\031\042" "\063\035\063\024"
-                           "\x09\x46\x13\x49\x4C\x7F\x11\xD1"
-                           "\x82\x22\x44\x45\x53\x54\x00\x00", 16+8+2);
+                            CAP_AIM_SERVERRELAY, 16+8+2);
 
   
   add_tlv(middletlvs, 0x0A, "\x00\x01", 2);
@@ -538,7 +576,9 @@ void v7_send_type1_msg2(V7Connection *conn, UIN_T uin, const gchar *msg)
 
 void v7_send_file_msg2(V7Connection *conn, UIN_T uin, const gchar *msg,
 		       WORD port, DWORD ip, gboolean filereq,
-		       const gchar *filename, DWORD filesize, gchar *msgid)
+		       const gchar *filename, DWORD filesize, gchar *msgid,
+		       WORD downcounter)
+		    
 {
   gchar *buffer;
   /* We have three levels of TLVS one within the other, its not nice at all,
@@ -569,26 +609,32 @@ void v7_send_file_msg2(V7Connection *conn, UIN_T uin, const gchar *msg,
   middletlvs = new_tlvstack("\0\0", 2);  /* might be 0002 for file-ack */
 
   add_nontlv(middletlvs, outertlvs->beg, 8); /* copy msgid */
-  add_nontlv(middletlvs,  "\x09\x46\x13\x49\x4C\x7F\x11\xD1"
-	     "\x82\x22\x44\x45\x53\x54\x00\x00", 16);
+  add_nontlv(middletlvs, CAP_AIM_SERVERRELAY, 16);
 
   if (filereq)
     add_tlv(middletlvs, 0x0A, "\x00\x01", 2);
   else
     add_tlv(middletlvs, 0x0A, "\x00\x02", 2);
-  add_tlv_w_be(middletlvs, 0x05, our_port);  /* port */
-  add_tlv_dw_be(middletlvs, 0x03, our_ip);
   add_tlv(middletlvs, 0x0F, NULL, 0);
-  
+  //add_tlv_dw_be(middletlvs, 0x03, our_ip);
+  add_tlv(middletlvs, 0x03, "\xa1\xb8\xa5\x2a", 4);
+  add_tlv_w_be(middletlvs, 0x05, port);  /* port */
+
   innertlvs = new_tlvstack("\x1B\0", 2);
   
   add_nontlv_w_le(innertlvs, 8); /* protocol version 8 */
   add_nontlv(innertlvs, zerobuf, 18);
   add_nontlv_dw_le(innertlvs, 0x03);
   add_nontlv(innertlvs, "\x04", 1); /* 1==firewall 4==normal */
-  add_nontlv_w_le(innertlvs, --conn->msg2_downcounter);
+  if (filereq)
+    add_nontlv_w_le(innertlvs, --conn->msg2_downcounter);
+  else
+    add_nontlv_w_le(innertlvs, downcounter);
   add_nontlv_w_be(innertlvs, 0x0E00);
-  add_nontlv_w_le(innertlvs, conn->msg2_downcounter);
+  if (filereq)
+    add_nontlv_w_le(innertlvs, conn->msg2_downcounter);
+  else
+    add_nontlv_w_le(innertlvs, downcounter);
   add_nontlv(innertlvs, zerobuf, 12);
   add_nontlv_w_le(innertlvs, 0x1A);
   /* first word of status code */
@@ -612,14 +658,23 @@ void v7_send_file_msg2(V7Connection *conn, UIN_T uin, const gchar *msg,
 
   add_nontlv_dw_le(ininnertlvs, strlen(msg));
   add_nontlv(ininnertlvs, msg, strlen(msg));
-  add_nontlv(ininnertlvs, zerobuf, 4); /* data I dont understand */
+  if (filereq)
+    add_nontlv (ininnertlvs, zerobuf, 4); /* data I dont understand */
+  else {
+    add_nontlv_w_be (ininnertlvs, port);
+    add_nontlv (ininnertlvs, zerobuf, 2);
+  }
   add_nontlv_lnts(ininnertlvs, filename);
   add_nontlv_dw_le(ininnertlvs, filesize);
-  add_nontlv_w_le(ininnertlvs, port);
-  add_nontlv(ininnertlvs, zerobuf, 2); /* data I dont understand */
+  if (filereq)
+    add_nontlv (ininnertlvs, zerobuf, 4); /* data I dont understand */
+  else {
+    add_nontlv_w_le(ininnertlvs, port);
+    add_nontlv (ininnertlvs, zerobuf, 2);
+  }
 
-  add_nontlv_dw_le(innertlvs, ininnertlvs->len+1);
-  add_nontlv(innertlvs, ininnertlvs->beg, ininnertlvs->len+1);
+  add_nontlv_dw_le(innertlvs, ininnertlvs->len);
+  add_nontlv(innertlvs, ininnertlvs->beg, ininnertlvs->len);
 
   free_tlvstack(ininnertlvs);
 
@@ -631,7 +686,7 @@ void v7_send_file_msg2(V7Connection *conn, UIN_T uin, const gchar *msg,
 
   free_tlvstack(middletlvs);
   
-  add_tlv(outertlvs, 3, NULL, 0); /* ACK request */
+  add_tlv (outertlvs, 3, NULL, 0); /* ACK request */
   
   snac_send(conn, outertlvs->beg, outertlvs->len ,FAMILY_04_MESSAGING,
             F04_CLIENT_SEND_MESSAGE, NULL, ANY_ID);
@@ -1159,13 +1214,19 @@ void v7_sendcontacts(V7Connection *conn, UIN_T uin, GList *contacts)
 
 void v7_iam_icq (V7Connection *conn)
 {
-  BYTE i_am_icq_array[40] = "\x00\x01\x00\x03\x00\x13\x00\x02"
-                            "\x00\x02\x00\x01\x00\x03\x00\x01"
-                            "\x00\x15\x00\x01\x00\x04\x00\x01"
-                            "\x00\x06\x00\x01\x00\x09\x00\x01"
-                            "\x00\x0A\x00\x01\x00\x0B\x00\x01";
+  BYTE family_ver[40] = \
+    "\x00\x01\x00\x03"
+    "\x00\x13\x00\x02"
+    "\x00\x02\x00\x01"
+    "\x00\x03\x00\x01"
+    "\x00\x15\x00\x01"
+    "\x00\x04\x00\x01"
+    "\x00\x06\x00\x01"
+    "\x00\x09\x00\x01"
+    "\x00\x0A\x00\x01"
+    "\x00\x0B\x00\x01";
 
-  snac_send(conn, i_am_icq_array, 40, FAMILY_01_GENERIC, F01_CLIENT_I_AM_ICQ,
+  snac_send(conn, family_ver, 40, FAMILY_01_GENERIC, F01_CLIENT_I_AM_ICQ,
             NULL, ANY_ID);
 }
 
@@ -1200,15 +1261,7 @@ void v7_make_requests(V7Connection *conn)
 void v7_set_user_info (V7Connection *conn)
 {
   TLVstack *tlv;
-  BYTE caps[64] =
-    /* Server relay */
-    "\x09\x46\x13\x49" "\x4C\x7F\x11\xD1" "\x82\x22\x44\x45" "\x53\x54\x00\x00"
-    /* RTF messages */
-    "\x97\xB1\x27\x51" "\x24\x3C\x43\x34" "\xAD\x22\xD6\xAB" "\xF7\x3F\x14\x92"
-    /* Unknown */
-    "\x2E\x7A\x64\x75" "\xFA\xDF\x4D\xC8" "\x88\x6F\xEA\x35" "\x95\xFD\xB6\xDF"
-    /* ICQ */
-    "\x09\x46\x13\x44" "\x4C\x7F\x11\xD1" "\x82\x22\x44\x45" "\x53\x54\x00\x00";
+  BYTE caps[16*4] = CAP_AIM_SERVERRELAY CAP_UTF8 CAP_RTFMSGS CAP_AIM_ISICQ;
 
   tlv = new_tlvstack(NULL,0);
   add_tlv(tlv, 5, caps, 64);
@@ -1265,16 +1318,6 @@ void v7_add_icbm (V7Connection *conn)
 
 void v7_client_ready (V7Connection *conn)
 {
-  /*  BYTE ready_data2000[64] = \
-    "\x00\x01\x00\x03\x01\x10\x02\x8A"
-    "\x00\x02\x00\x01\x01\x01\x02\x8A"
-    "\x00\x03\x00\x01\x01\x10\x02\x8A"
-    "\x00\x15\x00\x01\x01\x10\x02\x8A"
-    "\x00\x04\x00\x01\x01\x10\x02\x8A"
-    "\x00\x06\x00\x01\x01\x10\x02\x8A"
-    "\x00\x09\x00\x01\x01\x10\x02\x8A"
-    "\x00\x0A\x00\x01\x01\x10\x02\x8A";*/
-
   gchar ready_data2001[80] = \
     "\x00\x01\x00\x03\x01\x10\x04\x7B"
     "\x00\x13\x00\x02\x01\x10\x04\x7B"
@@ -1294,9 +1337,9 @@ void v7_client_ready (V7Connection *conn)
   enable_online_events = TRUE;
   gnomeicu_spinner_stop();
 
-  ready_set(FALSE);
+  gnomeicu_set_status_button (FALSE);
 
-  init();
+  gnomeicu_done_login ();
 
   v7_reqofflinemsg(conn);
 
@@ -1413,7 +1456,7 @@ void v7_sendfile(V7Connection *conn, UIN_T uin, const gchar *msg,
   }
 
   v7_send_file_msg2(conn, uin, msg, 0, conn->ourip, TRUE, xferinfo->filename, 
-		    xferinfo->total_bytes, xferinfo->msgid);
+		    xferinfo->total_bytes, xferinfo->msgid, 0);
 
   return;
 }
@@ -1429,13 +1472,12 @@ void v7_acceptfile(XferInfo *xfer)
     return;
 
   if (preferences_get_bool (PREFS_GNOMEICU_AUTO_ACCEPT_FILE)) {
-    if (xfer->auto_save_path)
-      g_free (xfer->auto_save_path);
+    g_free (xfer->auto_save_path);
 
     xfer->auto_save_path = preferences_get_string (PREFS_GNOMEICU_DOWNLOAD_PATH);
   }
 
   v7_send_file_msg2(xfer->conn, xfer->remote_contact->uin, xfer->msg, 
 		    xfer->port, xfer->conn->ourip, FALSE, xfer->filename,
-		    xfer->total_bytes, xfer->msgid);
+		    xfer->total_bytes, xfer->msgid, xfer->downcounter);
 }
