@@ -113,10 +113,6 @@ v7_request_contacts_list (V7Connection *conn)
 {
   guchar stamp[6];
 
-#ifdef TRACE_FUNCTION
-  g_print("v7_requestcontactslist\n");
-#endif
-
   /* request rights (0x13, 0x02) */
   snac_send (conn, NULL, 0, FAMILY_13_SAVED_LIST,
 	     F13_CLIENT_REQ_RIGHTS, NULL, ANY_ID);
@@ -162,13 +158,6 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
   guint32 timestamp;
   GSList *contact;
 
-  //gboolean force_migration = FALSE;
-
-
-#ifdef TRACE_FUNCTION
-  g_print("v7_read_contact_list\n");
-#endif
-
   Current_Status = STATUS_ONLINE;
 
   /* reset flags and counters so we can receive contact list again */
@@ -254,6 +243,10 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
         case 0x66: /* with auth */
           kontakt->wait_auth = TRUE;
           break;
+        case 0x13a: /* unknown tlv type, icq added after 2003a (jan 2004)*/
+	case 0x137: /* Users email, added by icqlite */
+	case 0x145: /* a message timestamp from icqlite */
+	  break;
         default:
           g_warning ("Unknown tlv type (0x%x) inside type 0", tlv2->type);
           break;
@@ -352,7 +345,7 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
     g_free (rec_str);
     rec_str = NULL;
     delete_tlv (tlv);
-  } /* end for(rec_cnt) */
+  } /* end for(rec_count) */
 
   /* read the 4 bytes timestamp at the end of the packet */
   timestamp = CharsBE_2_DW (here);
@@ -697,6 +690,9 @@ v7_snac13_check_srv_ack (V7Connection *conn, Snac *snac)
       } else {
         contactdata = contact->data;
       }
+      if (contact)
+	gnomeicu_tree_user_remove (contactdata);
+
       g_free(contactdata->uin);
       contactdata->uin = g_strdup (snac13_state.uin);
       g_free(contactdata->nick);
@@ -720,7 +716,8 @@ v7_snac13_check_srv_ack (V7Connection *conn, Snac *snac)
       v7_end_CL_edit (conn);
 
       /* refresh the gui */
-      gnomeicu_tree_user_add (g_slist_last (Contacts)->data);
+
+      gnomeicu_tree_user_add (contactdata);
 
       nick = contactdata->nick;
       break;
@@ -860,12 +857,11 @@ v7_try_add_uin (V7Connection *conn, UIN_T uin, const gchar *nick, guint gid,
  * and auth.c
  */
 void
-v7_ask_uin_for_auth (V7Connection *conn, gchar *authmsg)
+v7_ask_uin_for_auth (V7Connection *conn, gchar *authmsg, UIN_T uin)
 {
-  UIN_T uin;
   TLVstack *tlvs;
 
-  uin = g_strdup (snac13_state.uin);
+  // uin = g_strdup (snac13_state.uin);
 
   /* send the auth message first */
   tlvs = new_tlvstack (NULL, 0);
@@ -880,7 +876,8 @@ v7_ask_uin_for_auth (V7Connection *conn, gchar *authmsg)
 
   snac13_state.state = SNAC13_ADD_USER_AUTH;
   /* now add again, but with the awaiting auth bit set (TLV(0x66) */
-  v7_addto_contacts_list (conn, uin, snac13_state.nick, snac13_state.gid, TRUE);
+  if (snac13_state.nick)
+    v7_addto_contacts_list (conn, uin, snac13_state.nick, snac13_state.gid, TRUE);
 }
 
 /*
@@ -944,8 +941,7 @@ v7_recv_auth_request (Snac *snac)
   str = g_strndup (here, str_len);
 
   /* got all the info, now call the client auth handler */
-  auth_receive_request (uin, str);
-
+  authorization_request(uin, str, time(NULL));
   g_free (str);
 }
 
@@ -993,12 +989,14 @@ v7_user_grant_auth (Snac *snac)
   uin = g_strndup (here+1, here[0]);
   here = here + here[0] + 1;
 
+  /* FIXME this part should be in a seperate function */
   contact = Find_User (uin);
-
-  if (contact)
+  
+  if (contact) {
+    gnomeicu_tree_user_remove (kontakt);
     kontakt->wait_auth = FALSE;
-
-  /* TODO: need to add some GUI call after changing status */
+    gnomeicu_tree_user_add (kontakt);
+  }
 
   g_free (uin);
 }
