@@ -65,6 +65,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdio.h>
 
 #include "common.h"
 #include "gnomeicu.h"
@@ -375,7 +376,6 @@ xmlContactEndEle (XMLContactParseState *state, const xmlChar *name)
       g_free (passwd);
     }
     passwd = g_strdup (state->content->str);
-    /*    g_print ("got password = %s\n", passwd); */
     g_string_truncate (state->content, 0);
     break;
   case PARSER_ICQ_NICK:
@@ -602,6 +602,8 @@ xmlcontact_load (const gchar *filename)
   xmlParserCtxtPtr ctxt;
   XMLContactParseState state;
 
+  state.content=NULL;
+
   ctxt = xmlCreateFileParserCtxt (filename);
 
   if (!ctxt) {
@@ -628,6 +630,7 @@ xmlcontact_save (const gchar *filename)
   xmlDocPtr doc;
   xmlNsPtr icuNs;
   xmlNodePtr node;
+  gchar *tempfilename;
 
   doc = xmlNewDoc ("1.0");
 
@@ -641,9 +644,16 @@ xmlcontact_save (const gchar *filename)
   /* save icq contacts */
   xmlcontact_save_icq (doc->xmlRootNode);
 
-  if (xmlSaveFile (filename, doc) == -1) {
-    g_warning ("Cannot save XML contact file: %s", filename);
+  tempfilename =  g_strdup_printf("%s.tmp",filename);
+
+  if (xmlSaveFile (filename, doc) <= 0) {
+    g_warning ("Cannot save XML contact file: %s", tempfilename);
   }
+  else if (!rename (tempfilename, filename))
+    g_warning("Cannot save XML contacts into %s, but saved it in %s", filename,
+	      tempfilename);
+  
+  g_free(tempfilename);
 
   chmod(filename, S_IRUSR|S_IWUSR);
 
@@ -661,12 +671,22 @@ xmlcontact_save_icq (xmlNodePtr icuNode)
   gchar *tmpstr;
   GSList *tlist, *contact;
   gint contact_count = 0;
-
+  xmlChar *xmlEncoded;
+	
   icqNode = xmlNewChild (icuNode, NULL, "icq", NULL);
 
   xmlNewChild (icqNode, NULL, "uin", our_info->uin);
-  xmlNewChild (icqNode, NULL, "password", passwd);
-  xmlNewChild (icqNode, NULL, "nick", our_info->nick);
+	
+  xmlEncoded = xmlEncodeEntitiesReentrant (NULL, passwd);
+  xmlNewChild (icqNode, NULL, "password", xmlEncoded);
+  free (xmlEncoded);
+  xmlEncoded = NULL;
+	
+  xmlEncoded = xmlEncodeEntitiesReentrant (NULL, our_info->nick);
+  xmlNewChild (icqNode, NULL, "nick", xmlEncoded);
+  free (xmlEncoded);
+  xmlEncoded = NULL;
+
   xmlNewChild (icqNode, NULL, "require-auth", our_info->auth ? "true" : "false");
   xmlNewChild (icqNode, NULL, "webpresence", our_info->webpresence ? "true" : "false");
 
